from collections.abc import MutableMapping
from itertools import chain


class CaseInsensitiveDict(MutableMapping):
    def __init__(self, *args, **kwargs):
        self.store = dict()
        self.proxy = dict()

        self.update(dict(*args, **kwargs))  # use the free update to set keys

    def __contains__(self, k):
        return k.lower() in self.proxy

    def __delitem__(self, k):
        key = self.proxy[k.lower()]

        del self.store[key]
        del self.proxy[k.lower()]

    def __getitem__(self, k):
        key = self.proxy[k.lower()]
        return self.store[key]

    def get(self, k, default=None):
        key = self.proxy[k.lower()]
        return self.store[key] if key in self.store else default

    def __setitem__(self, k, v):
        self.store[k] = v
        self.proxy[k.lower()] = k

    def __iter__(self):
        return iter(self.store)

    def __len__(self):
        return len(self.store)


class DotDict(CaseInsensitiveDict):
    def __getattr__(obj, name):
        # Return {} if non-existent attr
        if name not in obj:
            return DotDict({})

        # python 3 val = dict.get(*args, None)
        val = obj.get(name)
        return DotDict(val) if type(val) is dict else val
        # return DotDict(val) if type(val) is dict else DotDict({args[1]: val})

    def __add__(self, other):
        if isinstance(other, str):
            return DotDict({k: v + other for k, v in self.items() if v})
        elif isinstance(other, DotDict):
            # python 3 return DotDict({**self, **other})
            new_dic = DotDict(self)
            new_dic.update(other)
            return new_dic

    def __radd__(self, other):
        if isinstance(other, str):
            return DotDict({k: other + v for k, v in self.items() if v})

    def __getitem__(self, key):
        try:
            return super(DotDict, self).__getitem__(key)
        except KeyError:
            return DotDict({})

    def __str__(self):
        return "\n".join(
            [
                "{}{} {}".format(k, "->" if isinstance(v, DotDict) else ":", v)
                for k, v in self.items()
            ]
        )



import sys
from collections import defaultdict
import threading

try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest

from wfuzz.fuzzobjects import FuzzWordType, FuzzType, FuzzPlugin

from .common import exec_banner, Term
from .getch import _Getch
from .output import getTerminalSize, wrap_always_list

usage = """\r\n
Interactive keyboard commands:\r\n
?: Show this help

p: Pause
s: Show stats
q: Cancel
"""


class SimpleEventDispatcher:
    def __init__(self):
        self.publisher = defaultdict(list)

    def create_event(self, msg):
        self.publisher[msg] = []

    def subscribe(self, func, msg, dynamic=False):
        if msg not in self.publisher and not dynamic:
            raise KeyError("subscribe. No such event: %s" % (msg))
        else:
            self.publisher[msg].append(func)

    def notify(self, msg, **event):
        if msg not in self.publisher:
            raise KeyError("notify. Event not subscribed: %s" % (msg,))
        else:
            for functor in self.publisher[msg]:
                functor(**event)


class KeyPress(threading.Thread):
    def __init__(self):
        threading.Thread.__init__(self)
        self.inkey = _Getch()
        self.setName("KeyPress")

        self.dispatcher = SimpleEventDispatcher()
        self.dispatcher.create_event("?")
        self.dispatcher.create_event("p")
        self.dispatcher.create_event("s")
        self.dispatcher.create_event("q")

        self.do_job = True

    def cancel_job(self):
        self.do_job = False

    def run(self):
        while self.do_job:
            k = self.inkey()
            if k and ord(k) == 3:
                self.dispatcher.notify("q", key="q")
            elif k == "p":
                self.dispatcher.notify("p", key="p")
            elif k == "s":
                self.dispatcher.notify("s", key="s")
            elif k == "?":
                self.dispatcher.notify("?", key="?")
            elif k == "q":
                self.dispatcher.notify("q", key="q")


class Controller:
    def __init__(self, fuzzer, view):
        self._debug = False
        self.fuzzer = fuzzer
        self.view = view
        self.__paused = False
        self.stats = fuzzer.options.get("compiled_stats")

        self.view.dispatcher.subscribe(self.on_help, "?")
        self.view.dispatcher.subscribe(self.on_pause, "p")
        self.view.dispatcher.subscribe(self.on_stats, "s")
        self.view.dispatcher.subscribe(self.on_exit, "q")

    # dynamic keyboard bindings
    def on_exit(self, **event):
        self.fuzzer.cancel_job()
        self.view.cancel_job()
        self.fuzzer.options.close()

    def on_help(self, **event):
        print(usage)

    def on_pause(self, **event):
        self.__paused = not self.__paused
        if self.__paused:
            self.fuzzer.pause_job()
        else:
            self.fuzzer.resume_job()

    def on_stats(self, **event):
        if self._debug:
            self.show_debug_stats()
        else:
            self.show_stats()

    def show_debug_stats(self):
        print("\n=============== Paused ==================")
        stats = self.fuzzer.stats()
        for k, v in list(stats.items()):
            print("%s: %s" % (k, v))
        print("\n=========================================")

    def show_stats(self):
        pending = self.stats.total_req - self.stats.processed()
        summary = self.stats
        summary.mark_end()
        print("\nTotal requests: %s\r" % str(summary.total_req))
        print("Pending requests: %s\r" % str(pending))

        if summary.backfeed() > 0:
            print(
                "Processed Requests: %s (%d + %d)\r"
                % (
                    str(summary.processed())[:8],
                    (summary.processed() - summary.backfeed()),
                    summary.backfeed(),
                )
            )
        else:
            print("Processed Requests: %s\r" % (str(summary.processed())[:8]))
        print("Filtered Requests: %s\r" % (str(summary.filtered())[:8]))
        req_sec = (
            summary.processed() / summary.totaltime if summary.totaltime > 0 else 0
        )
        print("Total time: %s\r" % str(summary.totaltime)[:8])
        if req_sec > 0:
            print("Requests/sec.: %s\r" % str(req_sec)[:8])
            eta = pending / req_sec
            if eta > 60:
                print("ET left min.: %s\r\n" % str(eta / 60)[:8])
            else:
                print("ET left sec.: %s\r\n" % str(eta)[:8])


class View:
    widths = [10, 8, 6, 8, 9, getTerminalSize()[0] - 65]
    verbose_widths = [10, 10, 8, 8, 6, 9, 30, 30, getTerminalSize()[0] - 145]

    def __init__(self, session_options):
        self.colour = session_options["colour"]
        self.verbose = session_options["verbose"]
        self.previous = session_options["previous"]
        self.term = Term()
        self.printed_lines = 0

    def _print_verbose(self, res, print_nres=True):
        txt_colour = (
            Term.noColour if not res.is_baseline or not self.colour else Term.fgCyan
        )
        if self.previous and self.colour and not print_nres:
            txt_colour = Term.fgCyan

        location = ""
        if "Location" in res.history.headers.response:
            location = res.history.headers.response["Location"]
        elif res.history.url != res.history.redirect_url:
            location = "(*) %s" % res.history.url

        server = ""
        if "Server" in res.history.headers.response:
            server = res.history.headers.response["Server"]

        rows = [
            ("%09d:" % res.nres if print_nres else " |_", txt_colour),
            ("%.3fs" % res.timer, txt_colour),
            (
                "%s" % "XXX" if res.exception else str(res.code),
                self.term.get_colour(res.code) if self.colour else txt_colour,
            ),
            ("%d L" % res.lines, txt_colour),
            ("%d W" % res.words, txt_colour),
            ("%d Ch" % res.chars, txt_colour),
            (server, txt_colour),
            (location, txt_colour),
            ('"%s"' % res.description, txt_colour),
        ]

        self.term.set_colour(txt_colour)
        self.printed_lines = self._print_line(rows, self.verbose_widths)

    def _print_header(self, rows, maxWidths):
        print("=" * (3 * len(maxWidths) + sum(maxWidths[:-1]) + 10))
        self._print_line(rows, maxWidths)
        print("=" * (3 * len(maxWidths) + sum(maxWidths[:-1]) + 10))
        print("")

    def _print_line(self, rows, maxWidths):
        def wrap_row(rows, maxWidths):
            newRows = [
                wrap_always_list(item[0], width) for item, width in zip(rows, maxWidths)
            ]
            return [[substr or "" for substr in item] for item in zip_longest(*newRows)]

        def print_row(row, rows):
            sys.stdout.write(
                "   ".join(
                    [
                        colour + str.ljust(str(item), width) + Term.reset
                        for (item, width, colour) in zip(
                            row, maxWidths, [colour[1] for colour in rows]
                        )
                    ]
                )
            )

        new_rows = wrap_row(rows, maxWidths)

        for row in new_rows:
            print_row(row, rows)
            sys.stdout.write("\n\r")

        sys.stdout.flush()
        return len(new_rows)

    def _print(self, res, print_nres=True):
        txt_colour = (
            Term.noColour if not res.is_baseline or not self.colour else Term.fgCyan
        )
        if self.previous and self.colour and not print_nres:
            txt_colour = Term.fgCyan

        rows = [
            ("%09d:" % res.nres if print_nres else " |_", txt_colour),
            (
                "%s" % "XXX" if res.exception else str(res.code),
                self.term.get_colour(res.code) if self.colour else txt_colour,
            ),
            ("%d L" % res.lines, txt_colour),
            ("%d W" % res.words, txt_colour),
            ("%d Ch" % res.chars, txt_colour),
            ('"%s"' % res.description, txt_colour),
        ]

        self.term.set_colour(txt_colour)
        self.printed_lines = self._print_line(rows, self.widths)

    def header(self, summary):
        print(exec_banner)
        if summary:
            print("Target: %s\r" % summary.url)
            if summary.total_req > 0:
                print("Total requests: %d\r\n" % summary.total_req)
            else:
                print("Total requests: <<unknown>>\r\n")

        if self.verbose:
            rows = [
                ("ID", Term.noColour),
                ("C.Time", Term.noColour),
                ("Response", Term.noColour),
                ("Lines", Term.noColour),
                ("Word", Term.noColour),
                ("Chars", Term.noColour),
                ("Server", Term.noColour),
                ("Redirect", Term.noColour),
                ("Payload", Term.noColour),
            ]

            widths = self.verbose_widths
        else:
            rows = [
                ("ID", Term.noColour),
                ("Response", Term.noColour),
                ("Lines", Term.noColour),
                ("Word", Term.noColour),
                ("Chars", Term.noColour),
                ("Payload", Term.noColour),
            ]

            widths = self.widths

        self._print_header(rows, widths)

    def result(self, res):
        if self.verbose:
            self._print_verbose(res)
        else:
            self._print(res)

        if not res.discarded:
            if (
                self.previous
                and res.payload_man
                and res.payload_man.get_payload_type(1) == FuzzWordType.FUZZRES
            ):
                prev_res = res.payload_man.get_payload_content(1)
                if self.verbose:
                    self._print_verbose(prev_res, print_nres=False)
                else:
                    self._print(prev_res, print_nres=False)

            if res.plugins_res:
                for plugin_res in res.plugins_res:
                    if not plugin_res.is_visible(self.verbose):
                        continue

                    sys.stdout.write(
                        " |_  {} {}\r".format(
                            plugin_res.issue, plugin_res.data if plugin_res.data else ""
                        )
                    )
                    sys.stdout.write(" |_  %s\r" % plugin_res.issue)
                    sys.stdout.write("\n\r")

            self.printed_lines = 0

        if self.printed_lines > 0:
            self.term.erase_lines(self.printed_lines + 1)

    def footer(self, summary):
        sys.stdout.write("\n\r")

        print(summary)



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.plugin_api.base import BasePayload
from wfuzz.fuzzobjects import FuzzWordType

import os

# Python 2 and 3: alternative 4
try:
    from urllib.parse import quote
except ImportError:
    from urllib import quote


@moduleman_plugin
class dirwalk(BasePayload):
    name = "dirwalk"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    description = (
        "Returns all the file paths found in the specified directory.",
        "Handy if you want to check a directory structure against a webserver,",
        "for example, because you have previously downloaded a specific version",
        "of what is supposed to be on-line.",
    )
    summary = "Returns filename's recursively from a local directory."
    category = ["default"]
    priority = 99

    parameters = (
        ("dir", "", True, "Directory path to walk and generate payload from."),
    )

    default_parameter = "dir"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        self.g = self._my_gen(self.params["dir"])

    def _my_gen(self, directory):
        for root, dirs, fnames in os.walk(directory):
            for f in fnames:
                relative_path = os.path.relpath(os.path.join(root, f), directory)
                yield quote(relative_path)

    def get_next(self):
        return next(self.g)

    def get_type(self):
        return FuzzWordType.WORD

    def count(self):
        return -1



# Plugins specializations with common methods useful for their own type
from wfuzz.plugin_api.urlutils import parse_url
from .base import BasePlugin


class DiscoveryPluginMixin:
    def queue_url(self, url):
        if not parse_url(url).isbllist:
            BasePlugin.queue_url(self, url)
            return True
        return False



import re

from wfuzz.plugin_api.base import BasePlugin
from wfuzz.exception import FuzzExceptPluginBadParams
from wfuzz.externals.moduleman.plugin import moduleman_plugin


@moduleman_plugin
class grep(BasePlugin):
    name = "grep"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "HTTP response grep"
    description = (
        "Extracts the given regex pattern from the HTTP response and prints it",
        "(It is not a filter operator)",
    )
    category = ["tools"]
    priority = 99

    parameters = (("regex", "", True, "Regex to perform the grep against."),)

    def __init__(self):
        BasePlugin.__init__(self)
        try:
            print(self.kbase["grep.regex"])
            self.regex = re.compile(
                self.kbase["grep.regex"][0], re.MULTILINE | re.DOTALL
            )
        except Exception:
            raise FuzzExceptPluginBadParams(
                "Incorrect regex or missing regex parameter."
            )

    def validate(self, fuzzresult):
        return True

    def process(self, fuzzresult):
        for r in self.regex.findall(fuzzresult.history.content):
            self.add_result("match", "Pattern match", r)



import re
import sys
import getopt
import warnings
from collections import defaultdict

from wfuzz.helpers.file_func import get_filter_help_file
from wfuzz.helpers.obj_dyn import allowed_fields
from wfuzz.filters.ppfilter import PYPARSING
from wfuzz.facade import Facade
from wfuzz.options import FuzzSession
from wfuzz.exception import FuzzException, FuzzExceptBadOptions, FuzzExceptBadInstall
from .common import help_banner, exec_banner
from .common import usage
from .common import brief_usage
from .common import verbose_usage
from wfuzz import __version__ as version
from .output import table_print

short_opts = "hLAZX:vcb:e:R:D:d:z:r:f:t:w:V:H:m:f:o:s:p:w:u:"
long_opts = [
    "efield=",
    "no-cache",
    "ee=",
    "zE=",
    "zD=",
    "field=",
    "ip=",
    "filter-help",
    "AAA",
    "AA",
    "slice=",
    "zP=",
    "oF=",
    "recipe=",
    "dump-recipe=",
    "req-delay=",
    "conn-delay=",
    "sc=",
    "sh=",
    "sl=",
    "sw=",
    "ss=",
    "hc=",
    "hh=",
    "hl=",
    "hw=",
    "hs=",
    "ntlm=",
    "basic=",
    "digest=",
    "follow",
    "script-help=",
    "script=",
    "script-args=",
    "prefilter=",
    "filter=",
    "interact",
    "help",
    "version",
    "dry-run",
    "prev",
]
REPEATABLE_OPTS = [
    "--efield",
    "--field",
    "--prefilter",
    "--recipe",
    "-z",
    "--zP",
    "--zD",
    "--slice",
    "payload",
    "-w",
    "-b",
    "-H",
    "-p",
]


class CLParser:
    def __init__(
        self,
        argv,
        short_opts=short_opts,
        long_opts=long_opts,
        help_banner=help_banner,
        brief_usage=brief_usage,
        verbose_usage=verbose_usage,
        usage=usage,
    ):
        self.argv = argv
        self.short_opts = short_opts
        self.long_opts = long_opts
        self.help_banner = help_banner
        self.brief_usage = brief_usage
        self.verbose_usage = verbose_usage
        self.usage = usage

    def show_brief_usage(self):
        print(self.help_banner)
        print(self.brief_usage)

    def show_verbose_usage(self):
        print(self.help_banner)
        print(self.verbose_usage)

    def show_usage(self):
        print(self.help_banner)
        print(self.usage)

    def show_plugins_help(self, registrant, cols=3, category="$all$"):
        print("\nAvailable %s:\n" % registrant)
        table_print(
            [x[cols:] for x in Facade().proxy(registrant).get_plugins_ext(category)]
        )
        sys.exit(0)

    def show_plugins_names(self, registrant):
        print("\n".join(Facade().proxy(registrant).get_plugins_names("$all$")))

    def show_plugin_ext_help(self, registrant, category="$all$"):
        for plugin in Facade().proxy(registrant).get_plugins(category):
            print("Name: %s %s" % (plugin.name, plugin.version))
            print("Categories: %s" % ",".join(plugin.category))
            print("Summary: %s" % plugin.summary)
            print("Author: %s" % ",".join(plugin.author))
            print("Description:")
            for desc_lines in plugin.description:
                print("   %s" % desc_lines)
            print("Parameters:")
            for name, default_value, mandatory, description in plugin.parameters:
                print(
                    "   {} {}{}: {}".format(
                        "+" if mandatory else "-",
                        name,
                        " (= %s)" % str(default_value)
                        if default_value is not None
                        else "",
                        description,
                    )
                )
            print("\n")

        sys.exit(0)

    def parse_cl(self):
        # Usage and command line help
        try:
            opts, args = getopt.getopt(self.argv[1:], self.short_opts, self.long_opts)
            optsd = defaultdict(list)

            payload_cache = {}
            for i, j in opts:
                if i in ["-z", "--zP", "--slice", "-w", "--zD", "--zE"]:
                    if i in ["-z", "-w"]:
                        if payload_cache:
                            optsd["payload"].append(payload_cache)
                            payload_cache = {}

                    payload_cache[i] = j
                optsd[i].append(j)

            if not args and not optsd:
                self.show_brief_usage()
                sys.exit(1)

            if payload_cache:
                optsd["payload"].append(payload_cache)
                payload_cache = {}

            self._parse_help_opt(optsd)

            url = None
            if len(args) == 1:
                url = args[0]
            elif len(args) > 1:
                raise FuzzExceptBadOptions("Too many arguments.")

            options = FuzzSession()

            cli_url = None
            if "-u" in optsd:
                if (url is not None and url != "FUZZ") or url == optsd["-u"][0]:
                    raise FuzzExceptBadOptions(
                        "Specify the URL either with -u or last argument. If you want to use a full payload, it can only be specified with FUZZ."
                    )

                cli_url = optsd["-u"][0]

            if cli_url:
                url = cli_url

            # check command line options correctness
            self._check_options(optsd)

            # parse options from recipe first
            if "--recipe" in optsd:
                for recipe in optsd["--recipe"]:
                    options.import_from_file(recipe)

            # command line has priority over recipe
            self._parse_options(optsd, options)
            self._parse_conn_options(optsd, options)
            self._parse_filters(optsd, options)
            self._parse_seed(url, optsd, options)
            self._parse_payload(optsd, options)
            self._parse_scripts(optsd, options)

            if "--dump-recipe" in optsd:
                print(exec_banner)

                for error_msg in options.validate():
                    warnings.warn("Recipe parsing error: {}".format(error_msg))

                print("")

                options.export_to_file(optsd["--dump-recipe"][0])
                print("Recipe written to %s." % (optsd["--dump-recipe"][0],))
                sys.exit(0)

            return options
        except FuzzException as e:
            self.show_brief_usage()
            raise e
        except ValueError:
            self.show_brief_usage()
            raise FuzzExceptBadOptions("Incorrect options, please check help.")
        except getopt.GetoptError as qw:
            self.show_brief_usage()
            raise FuzzExceptBadOptions("%s." % str(qw))

    def _parse_help_opt(self, optsd):
        if "--version" in optsd:
            print(version)
            sys.exit(0)

        if "-h" in optsd:
            self.show_usage()
            sys.exit(0)

        if "--help" in optsd:
            self.show_verbose_usage()
            sys.exit(0)

        if "--filter-help" in optsd:
            FILTER_HELP_REGEX_EXP = (
                "Filter Language\n---------------\n\n(.*?)Filtering results"
            )
            FILTER_HELP_REGEX = re.compile(
                FILTER_HELP_REGEX_EXP, re.MULTILINE | re.DOTALL
            )

            print(FILTER_HELP_REGEX.search(get_filter_help_file()).group(1))

            sys.exit(0)

        # Extensions help
        if "--script-help" in optsd:
            script_string = optsd["--script-help"][0]
            if script_string == "":
                script_string = "$all$"

            self.show_plugin_ext_help("scripts", category=script_string)

        if "--ee" in optsd:
            if "payloads" in optsd["--ee"]:
                self.show_plugins_names("payloads")
            elif "encoders" in optsd["--ee"]:
                self.show_plugins_names("encoders")
            elif "iterators" in optsd["--ee"]:
                self.show_plugins_names("iterators")
            elif "printers" in optsd["--ee"]:
                self.show_plugins_names("printers")
            elif "scripts" in optsd["--ee"]:
                self.show_plugins_names("scripts")
            elif "fields" in optsd["--ee"]:
                print("\n".join(allowed_fields))
            elif "files" in optsd["--ee"]:
                print("\n".join(Facade().sett.get("general", "lookup_dirs").split(",")))
            elif "registrants" in optsd["--ee"]:
                print("\n".join(Facade().get_registrants()))
            elif "options" in optsd["--ee"]:
                print(
                    "\n".join(
                        ["-{}".format(opt) for opt in self.short_opts.replace(":", "")]
                    )
                )
                print(
                    "\n".join(
                        ["--{}".format(opt.replace("=", "")) for opt in self.long_opts]
                    )
                )
            else:
                raise FuzzExceptBadOptions(
                    "Unknown category. Valid values are: payloads, encoders, iterators, printers or scripts."
                )
            sys.exit(0)

        if "-e" in optsd:
            if "payloads" in optsd["-e"]:
                self.show_plugins_help("payloads")
            elif "encoders" in optsd["-e"]:
                self.show_plugins_help("encoders", 2)
            elif "iterators" in optsd["-e"]:
                self.show_plugins_help("iterators")
            elif "printers" in optsd["-e"]:
                self.show_plugins_help("printers")
            elif "scripts" in optsd["-e"]:
                self.show_plugins_help("scripts", 2)
            else:
                raise FuzzExceptBadOptions(
                    "Unknown category. Valid values are: payloads, encoders, iterators, printers or scripts."
                )

        if "-f" in optsd:
            if "help" in optsd["-f"]:
                self.show_plugins_help("printers")
        if "-o" in optsd:
            if "help" in optsd["-o"]:
                self.show_plugins_help("printers")
        if "-m" in optsd:
            if "help" in optsd["-m"]:
                self.show_plugins_help("iterators")
        if "-z" in optsd:
            if "help" in optsd["-z"]:
                filt = optsd["--slice"][0] if "--slice" in optsd else "$all$"
                self.show_plugin_ext_help("payloads", category=filt)

    def _check_options(self, optsd):
        # Check for repeated flags
        opt_list = [i for i in optsd if i not in REPEATABLE_OPTS and len(optsd[i]) > 1]
        if opt_list:
            raise FuzzExceptBadOptions(
                "Bad usage: Only one %s option could be specified at the same time."
                % " ".join(opt_list)
            )

        # -A and script not allowed at the same time
        if "--script" in list(optsd.keys()) and [
            key for key in optsd.keys() if key in ["-A", "--AA", "--AAA"]
        ]:
            raise FuzzExceptBadOptions(
                "Bad usage: --scripts and -A, --AA, --AAA are incompatible options."
            )

        if "-s" in list(optsd.keys()) and "-t" in list(optsd.keys()):
            warnings.warn(
                "When using delayed requests concurrent requests are limited to 1, therefore the -s switch will be ignored."
            )

    def _parse_filters(self, optsd, filter_params):
        """
        filter_params = dict(
            hs = None,
            hc = [],
            hw = [],
            hl = [],
            hh = [],
            ss = None,
            sc = [],
            sw = [],
            sl = [],
            sh = [],
            filter = "",
            prefilter = "",
            ),
        """

        if "--prefilter" in optsd:
            if not PYPARSING:
                raise FuzzExceptBadInstall("--prefilter switch needs pyparsing module.")

            for prefilter_opt in optsd["--prefilter"]:
                filter_params["prefilter"].append(prefilter_opt)

        if "--filter" in optsd:
            if not PYPARSING:
                raise FuzzExceptBadInstall("--filter switch needs pyparsing module.")
            filter_params["filter"] = optsd["--filter"][0]

        if "--hc" in optsd:
            filter_params["hc"] = optsd["--hc"][0].split(",")
        if "--hw" in optsd:
            filter_params["hw"] = optsd["--hw"][0].split(",")
        if "--hl" in optsd:
            filter_params["hl"] = optsd["--hl"][0].split(",")
        if "--hh" in optsd:
            filter_params["hh"] = optsd["--hh"][0].split(",")
        if "--hs" in optsd:
            filter_params["hs"] = optsd["--hs"][0]
        if "--sc" in optsd:
            filter_params["sc"] = optsd["--sc"][0].split(",")
        if "--sw" in optsd:
            filter_params["sw"] = optsd["--sw"][0].split(",")
        if "--sl" in optsd:
            filter_params["sl"] = optsd["--sl"][0].split(",")
        if "--sh" in optsd:
            filter_params["sh"] = optsd["--sh"][0].split(",")
        if "--ss" in optsd:
            filter_params["ss"] = optsd["--ss"][0]

    def _parse_payload(self, optsd, options):
        """
        options = dict(
            payloads = [],
            iterator = None,
        )
        """

        payloads_list = []

        for payload in optsd["payload"]:
            if "-z" not in payload and "-w" not in payload:
                raise FuzzExceptBadOptions(
                    "--zP and --slice must be preceded by a -z or -w switch."
                )

            zpayl = payload["-z"] if "-z" in payload else "file,%s" % payload["-w"]
            extraparams = payload["--zP"] if "--zP" in payload else None
            sliceit = payload["--slice"] if "--slice" in payload else None

            vals = zpayl.split(",")

            default_param = None
            params = {}

            if len(vals) >= 2:
                name, default_param = vals[:2]
            else:
                name = vals[0]

            default_param_cli = payload["--zD"] if "--zD" in payload else None
            if default_param_cli and default_param:
                raise FuzzExceptBadOptions("--zD and -z parameters are exclusive.")
            elif default_param_cli:
                default_param = default_param_cli

            if extraparams:
                params = dict([x.split("=", 1) for x in extraparams.split(",")])
            if default_param:
                params["default"] = default_param

            encoders = vals[2] if len(vals) == 3 else None
            encoders_cli = payload["--zE"] if "--zE" in payload else None
            if encoders_cli and encoders:
                raise FuzzExceptBadOptions("--zE and -z encoders are exclusive.")
            elif encoders_cli:
                encoders = encoders_cli

            if encoders:
                params["encoder"] = encoders.split("-")
            elif "encoder" in params:
                params["encoder"] = params["encoder"].split("-")
            else:
                params["encoder"] = None

            payloads_list.append((name, params, sliceit))

        if "-m" in optsd:
            options["iterator"] = optsd["-m"][0]

        if payloads_list:
            options["payloads"] = payloads_list

    def _parse_seed(self, url, optsd, options):
        if url:
            options["url"] = url

        if "-X" in optsd:
            options["method"] = optsd["-X"][0]

        if "--basic" in optsd:
            options["auth"] = {"method": "basic", "credentials": optsd["--basic"][0]}

        if "--digest" in optsd:
            options["auth"] = {"method": "digest", "credentials": optsd["--digest"][0]}

        if "--ntlm" in optsd:
            options["auth"] = {"method": "ntlm", "credentials": optsd["--ntlm"][0]}

        if "--follow" in optsd or "-L" in optsd:
            options["follow"] = True

        if "--field" in optsd:
            for field in optsd["--field"]:
                options["fields"].append(field)
            options["show_field"] = True
        elif "--efield" in optsd:
            for field in optsd["--efield"]:
                options["fields"].append(field)

            options["show_field"] = False
        else:
            options["show_field"] = None

        if "--ip" in optsd:
            splitted = optsd["--ip"][0].partition(":")
            if not splitted[0]:
                raise FuzzExceptBadOptions("An IP must be specified")

            options["connect_to_ip"] = {
                "ip": splitted[0],
                "port": splitted[2] if splitted[2] else "80",
            }

        if "-d" in optsd:
            options["postdata"] = optsd["-d"][0]

        for bb in optsd["-b"]:
            options["cookie"].append(bb)

        for x in optsd["-H"]:
            splitted = x.partition(":")
            if splitted[1] != ":":
                raise FuzzExceptBadOptions(
                    'Wrong header specified, it should be in the format "name: value".'
                )
            options["headers"].append((splitted[0], splitted[2].strip()))

        if "-V" in optsd:
            options["allvars"] = str(optsd["-V"][0])

    def _parse_conn_options(self, optsd, conn_options):
        if "-p" in optsd:
            proxy = []

            for p in optsd["-p"]:
                vals = p.split(":")

                if len(vals) == 2:
                    proxy.append((vals[0], vals[1], "HTTP"))
                elif len(vals) == 3:
                    proxy.append((vals[0], vals[1], vals[2]))
                else:
                    raise FuzzExceptBadOptions("Bad proxy parameter specified.")

            conn_options["proxies"] = proxy

        if "--conn-delay" in optsd:
            conn_options["conn_delay"] = int(optsd["--conn-delay"][0])

        if "--req-delay" in optsd:
            conn_options["req_delay"] = int(optsd["--req-delay"][0])

        if "-R" in optsd:
            conn_options["rlevel"] = int(optsd["-R"][0])

        if "-D" in optsd:
            conn_options["dlevel"] = int(optsd["-D"][0])

        if "-Z" in optsd:
            conn_options["scanmode"] = True

        if "-s" in optsd:
            conn_options["delay"] = float(optsd["-s"][0])

        if "-t" in optsd:
            conn_options["concurrent"] = int(optsd["-t"][0])

    def _parse_options(self, optsd, options):
        if "--oF" in optsd:
            options["save"] = optsd["--oF"][0]

        if "-v" in optsd:
            options["verbose"] = True

        if "--prev" in optsd:
            options["previous"] = True

        if "--no-cache" in optsd:
            options["no_cache"] = True

        if "-c" in optsd:
            options["colour"] = True

        if [key for key in optsd.keys() if key in ["-A", "--AA", "--AAA"]]:
            options["verbose"] = True
            options["colour"] = True

        if "-f" in optsd:
            vals = optsd["-f"][0].split(",", 1)

            if len(vals) == 1:
                options["printer"] = (vals[0], None)
            else:
                options["printer"] = vals

        if "-o" in optsd:
            options["console_printer"] = optsd["-o"][0]

        if "--recipe" in optsd:
            options["recipe"] = optsd["--recipe"]

        if "--dry-run" in optsd:
            options["transport"] = "dryrun"

        if "--interact" in optsd:
            options["interactive"] = True

    def _parse_scripts(self, optsd, options):
        """
        options = dict(
            script = "",
            script_args = {},
        )
        """

        if "-A" in optsd:
            options["script"] = "default"

        if "--AA" in optsd:
            options["script"] = "default,verbose"

        if "--AAA" in optsd:
            options["script"] = "default,verbose,discovery"

        if "--script" in optsd:
            options["script"] = (
                "default" if optsd["--script"][0] == "" else optsd["--script"][0]
            )

        if "--script-args" in optsd:
            try:
                options["script_args"] = dict(
                    [x.split("=", 1) for x in optsd["--script-args"][0].split(",")]
                )
            except ValueError:
                raise FuzzExceptBadOptions(
                    "Script arguments: Incorrect arguments format supplied."
                )



from wfuzz.fuzzobjects import FuzzWord, FuzzPlugin
from wfuzz.exception import (
    FuzzExceptBadFile,
    FuzzExceptBadOptions,
    FuzzExceptPluginError,
)
from wfuzz.facade import Facade
from wfuzz.factories.plugin_factory import plugin_factory
from wfuzz.helpers.file_func import find_file_in_paths

import sys
import os
from distutils import util

# python 2 and 3: iterator
from builtins import object


# Util methods for accessing search results
class BasePlugin:
    def __init__(self):
        self.results_queue = None
        self.base_fuzz_res = None

        # check mandatory params, assign default values
        for name, default_value, required, description in self.parameters:
            param_name = "{}.{}".format(self.name, name)

            if required and param_name not in list(self.kbase.keys()):
                raise FuzzExceptBadOptions(
                    "Plugins, missing parameter %s!" % (param_name,)
                )

            if param_name not in list(self.kbase.keys()):
                self.kbase[param_name] = default_value

    def run(self, fuzzresult, control_queue, results_queue):
        try:
            self.results_queue = results_queue
            self.base_fuzz_res = fuzzresult
            self.process(fuzzresult)
        except Exception as e:
            results_queue.put(plugin_factory.create("plugin_from_error", self.name, e))
        finally:
            control_queue.get()
            control_queue.task_done()
            return

    def process(self, fuzzresult):
        """
        This is were the plugin processing is done. Any wfuzz plugin must implement this method, do its job with the fuzzresult received and:
        - queue_url: if it is a discovery plugin enqueing more HTTP request that at some point will generate more results
        - add_result: Add information about the obtained results after the processing with an accurate description

        A kbase (get_kbase, has_kbase, add_kbase) is shared between all plugins. this can be used to store and retrieve relevant "collaborative" information.
        """
        raise NotImplementedError

    def validate(self):
        raise FuzzExceptPluginError("Method count not implemented")

    def add_result(self, itype, issue, data, severity=FuzzPlugin.INFO):
        self.results_queue.put(
            plugin_factory.create(
                "plugin_from_finding", self.name, itype, issue, data, severity
            )
        )

    def queue_url(self, url):
        self.results_queue.put(
            plugin_factory.create(
                "plugin_from_recursion", self.name, self.base_fuzz_res, url
            )
        )

    def _bool(self, value):
        return bool(util.strtobool(value))


class BasePrinter:
    def __init__(self, output):
        self.f = None
        if output:
            try:
                self.f = open(output, "w")
            except IOError as e:
                raise FuzzExceptBadFile("Error opening file. %s" % str(e))
        else:
            self.f = sys.stdout

        self.verbose = Facade().printers.kbase["verbose"]

    def header(self):
        raise FuzzExceptPluginError("Method header not implemented")

    def footer(self):
        raise FuzzExceptPluginError("Method footer not implemented")

    def result(self):
        raise FuzzExceptPluginError("Method result not implemented")


class BasePayload(object):
    def __init__(self, params):
        self.params = params

        # default params
        if "default" in self.params:
            self.params[self.default_parameter] = self.params["default"]

            if not self.default_parameter:
                raise FuzzExceptBadOptions("Too many plugin parameters specified")

        # Check for allowed parameters
        if [
            k
            for k in list(self.params.keys())
            if k not in [x[0] for x in self.parameters]
            and k not in ["encoder", "default"]
        ]:
            raise FuzzExceptBadOptions(
                "Plugin %s, unknown parameter specified!" % (self.name)
            )

        # check mandatory params, assign default values
        for name, default_value, required, description in self.parameters:
            if required and name not in self.params:
                raise FuzzExceptBadOptions(
                    "Plugin %s, missing parameter %s!" % (self.name, name)
                )

            if name not in self.params:
                self.params[name] = default_value

    def get_type(self):
        raise FuzzExceptPluginError("Method get_type not implemented")

    def get_next(self):
        raise FuzzExceptPluginError("Method get_next not implemented")

    def __next__(self):
        return FuzzWord(self.get_next(), self.get_type())

    def count(self):
        raise FuzzExceptPluginError("Method count not implemented")

    def __iter__(self):
        return self

    def close(self):
        pass

    def find_file(self, name):
        if os.path.exists(name):
            return name

        for pa in Facade().sett.get("general", "lookup_dirs").split(","):
            fn = find_file_in_paths(name, pa)

            if fn is not None:
                return fn

        return name



import functools
from .obj_dic import DotDict


allowed_fields = [
    "description",
    "nres",
    "code",
    "chars",
    "lines",
    "words",
    "md5",
    "l",
    "h",
    "w",
    "c",
    "history",
    "plugins",
    "url",
    "content",
    "history.url",
    "history.method",
    "history.scheme",
    "history.host",
    "history.content",
    "history.raw_content" "history.is_path",
    "history.pstrip",
    "history.cookies",
    "history.headers",
    "history.params",
    "r",
    "r.reqtime",
    "r.url",
    "r.method",
    "r.scheme",
    "r.host",
    "r.content",
    "r.raw_content" "r.is_path",
    "r.pstrip",
    "r.cookies.",
    "r.headers.",
    "r.params.",
]


def _check_allowed_field(attr):
    if [field for field in allowed_fields if attr.startswith(field)]:
        return True
    return False


def _get_alias(attr):
    attr_alias = {
        "l": "lines",
        "h": "chars",
        "w": "words",
        "c": "code",
        "r": "history",
    }

    if attr in attr_alias:
        return attr_alias[attr]

    return attr


def rsetattr(obj, attr, new_val, operation):
    # if not _check_allowed_field(attr):
    #    raise AttributeError("Unknown field {}".format(attr))

    pre, _, post = attr.rpartition(".")

    pre_post = None
    if len(attr.split(".")) > 3:
        pre_post = post
        pre, _, post = pre.rpartition(".")

    post = _get_alias(post)

    try:
        obj_to_set = rgetattr(obj, pre) if pre else obj
        prev_val = rgetattr(obj, attr)
        if pre_post is not None:
            prev_val = DotDict({pre_post: prev_val})

        if operation is not None:
            val = operation(prev_val, new_val)
        else:
            if isinstance(prev_val, DotDict):
                val = {k: new_val for k, v in prev_val.items()}
            else:
                val = new_val

        return setattr(obj_to_set, post, val)
    except AttributeError:
        raise AttributeError(
            "rsetattr: Can't set '{}' attribute of {}.".format(
                post, obj_to_set.__class__
            )
        )


def rgetattr(obj, attr, *args):
    def _getattr(obj, attr):
        attr = _get_alias(attr)
        try:
            return getattr(obj, attr, *args)
        except AttributeError:
            raise AttributeError(
                "rgetattr: Can't get '{}' attribute from '{}'.".format(
                    attr, obj.__class__
                )
            )

    # if not _check_allowed_field(attr):
    # raise AttributeError("Unknown field {}".format(attr))

    return functools.reduce(_getattr, [obj] + attr.split("."))



from ..exception import FuzzExceptBadOptions

import re
import collections

from ..facade import BASELINE_CODE


class FuzzResSimpleFilter:
    def __init__(self, ffilter=None):
        self.hideparams = dict(
            regex_show=None,
            codes_show=None,
            codes=[],
            words=[],
            lines=[],
            chars=[],
            regex=None,
        )

        if ffilter is not None:
            self.hideparams = ffilter

        self.stack = []

        self._cache = collections.defaultdict(set)

    def is_active(self):
        return any(
            [
                self.hideparams["regex_show"] is not None,
                self.hideparams["codes_show"] is not None,
            ]
        )

    def set_baseline(self, res):
        if BASELINE_CODE in self.hideparams["lines"]:
            self.hideparams["lines"].append(res.lines)
        if BASELINE_CODE in self.hideparams["codes"]:
            self.hideparams["codes"].append(res.code)
        if BASELINE_CODE in self.hideparams["words"]:
            self.hideparams["words"].append(res.words)
        if BASELINE_CODE in self.hideparams["chars"]:
            self.hideparams["chars"].append(res.chars)

    def is_visible(self, res):
        if self.hideparams["codes_show"] is None:
            cond1 = True
        else:
            cond1 = not self.hideparams["codes_show"]

        if self.hideparams["regex_show"] is None:
            cond2 = True
        else:
            cond2 = not self.hideparams["regex_show"]

        if (
            res.code in self.hideparams["codes"]
            or res.lines in self.hideparams["lines"]
            or res.words in self.hideparams["words"]
            or res.chars in self.hideparams["chars"]
        ):
            cond1 = self.hideparams["codes_show"]

        if self.hideparams["regex"]:
            if self.hideparams["regex"].search(res.history.content):
                cond2 = self.hideparams["regex_show"]

        return cond1 and cond2

    @staticmethod
    def from_options(filter_options):
        ffilter = FuzzResSimpleFilter()

        try:
            if filter_options["ss"] is not None:
                ffilter.hideparams["regex_show"] = True
                ffilter.hideparams["regex"] = re.compile(
                    filter_options["ss"], re.MULTILINE | re.DOTALL
                )

            elif filter_options["hs"] is not None:
                ffilter.hideparams["regex_show"] = False
                ffilter.hideparams["regex"] = re.compile(
                    filter_options["hs"], re.MULTILINE | re.DOTALL
                )
        except Exception as e:
            raise FuzzExceptBadOptions(
                "Invalid regex expression used in filter: %s" % str(e)
            )

        if [x for x in ["sc", "sw", "sh", "sl"] if len(filter_options[x]) > 0]:
            ffilter.hideparams["codes_show"] = True
            ffilter.hideparams["codes"] = filter_options["sc"]
            ffilter.hideparams["words"] = filter_options["sw"]
            ffilter.hideparams["lines"] = filter_options["sl"]
            ffilter.hideparams["chars"] = filter_options["sh"]
        elif [x for x in ["hc", "hw", "hh", "hl"] if len(filter_options[x]) > 0]:
            ffilter.hideparams["codes_show"] = False
            ffilter.hideparams["codes"] = filter_options["hc"]
            ffilter.hideparams["words"] = filter_options["hw"]
            ffilter.hideparams["lines"] = filter_options["hl"]
            ffilter.hideparams["chars"] = filter_options["hh"]

        return ffilter



import re
import abc

from ..helpers.obj_dyn import (
    rgetattr,
    rsetattr,
)
from ..exception import FuzzExceptBadOptions


class Singleton(type):
    """ Singleton metaclass. Use by defining the metaclass of a class Singleton,
        e.g.: class ThereCanBeOnlyOne:
                  __metaclass__ = Singleton
    """

    def __call__(class_, *args, **kwargs):
        if not class_.hasInstance():
            class_.instance = super(Singleton, class_).__call__(*args, **kwargs)
        return class_.instance

    def deleteInstance(class_):
        """ Delete the (only) instance. This method is mainly for unittests so
            they can start with a clean slate. """
        if class_.hasInstance():
            del class_.instance

    def hasInstance(class_):
        """ Has the (only) instance been created already? """
        return hasattr(class_, "instance")


class ObjectFactory:
    def __init__(self, builders):
        self._builders = builders

    def create(self, key, *args, **kwargs):
        builder = self._builders.get(key)
        if not builder:
            raise ValueError(key)
        return builder(*args, **kwargs)


class HttpRequestFactory(abc.ABC):
    @staticmethod
    @abc.abstractmethod
    def to_http_object(options, to_http, from_req):
        pass

    @staticmethod
    @abc.abstractmethod
    def from_http_object(options, from_http, raw_header, raw_body):
        pass


class SeedBuilderHelper:
    FUZZ_MARKERS_REGEX = re.compile(
        r"(?P<full_marker>(?P<word>FUZ(?P<index>\d)*Z)(?P<nonfuzz_marker>(?:\[(?P<field>.*?)\])?(?P<full_bl>\{(?P<bl_value>.*?)\})?))"
    )
    REQ_ATTR = ["raw_request", "scheme", "method", "auth.credentials"]

    @staticmethod
    def _get_markers(text):
        return [
            m.groupdict() for m in SeedBuilderHelper.FUZZ_MARKERS_REGEX.finditer(text)
        ]

    @staticmethod
    def get_marker_dict(freq):
        marker_dict_list = []

        for text in [rgetattr(freq, field) for field in SeedBuilderHelper.REQ_ATTR]:
            marker_dict_list += SeedBuilderHelper._get_markers(text)

        # validate
        if len({bd["bl_value"] is None for bd in marker_dict_list}) > 1:
            raise FuzzExceptBadOptions(
                "You must supply a baseline value per FUZZ word."
            )

        return marker_dict_list

    @staticmethod
    def _remove_markers(freq, markers, mark_name):
        scheme = freq.scheme
        for mark in [
            mark[mark_name] for mark in markers if mark[mark_name] is not None
        ]:
            for field in SeedBuilderHelper.REQ_ATTR:
                old_value = rgetattr(freq, field)
                new_value = old_value.replace(mark, "")

                if field == "raw_request":
                    freq.update_from_raw_http(new_value, scheme)
                else:
                    rsetattr(freq, field, new_value, None)

    @staticmethod
    def remove_baseline_markers(freq, markers):
        SeedBuilderHelper._remove_markers(freq, markers, "full_bl")
        return freq

    @staticmethod
    def remove_nonfuzz_markers(freq, markers):
        SeedBuilderHelper._remove_markers(markers, "nonfuzz_marker")
        return freq

    # Not working due to reqresp internals
    # def replace_markers(self, seed, fpm):
    #     for payload in fpm.get_payloads():
    #         for field in self.REQ_ATTR:
    #             old_value = rgetattr(seed, field)
    #             new_value = old_value.replace(payload.marker, payload.value)
    #             rsetattr(seed, field, new_value , None)

    @staticmethod
    def replace_markers(freq, fpm):
        rawReq = str(freq)
        rawUrl = freq.redirect_url
        scheme = freq.scheme
        old_auth = freq.auth

        for payload in [
            payload for payload in fpm.get_payloads() if payload.marker is not None
        ]:
            if old_auth.method:
                old_auth["credentials"] = old_auth["credentials"].replace(
                    payload.marker, str(payload.value)
                )
            rawUrl = rawUrl.replace(payload.marker, str(payload.value))
            rawReq = rawReq.replace(payload.marker, str(payload.value))
            scheme = scheme.replace(payload.marker, str(payload.value))

        freq.update_from_raw_http(rawReq, scheme)
        freq.url = rawUrl
        if old_auth.method:
            freq.auth = old_auth

        return freq



# Python 2 and 3
try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

from wfuzz.plugin_api.mixins import DiscoveryPluginMixin
from wfuzz.plugin_api.base import BasePlugin
from wfuzz.exception import FuzzExceptResourceParseError
from wfuzz.externals.moduleman.plugin import moduleman_plugin

import tempfile
import sqlite3


@moduleman_plugin
class wcdb_extractor(BasePlugin, DiscoveryPluginMixin):
    name = "wc_extractor"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "Parses subversion's wc.db file."
    description = ("Parses subversion's wc.db file.",)
    category = ["active", "discovery"]
    priority = 99

    parameters = ()

    def __init__(self):
        BasePlugin.__init__(self)

    def validate(self, fuzzresult):
        return fuzzresult.url.find(".svn/wc.d") > 0 and fuzzresult.code == 200

    def readwc(self, content):
        """
        Function shamesly copied (and adapted) from https://github.com/anantshri/svn-extractor/
        Credit (C) Anant Shrivastava http://anantshri.info
        """
        author_list = []
        list_items = None
        (fd, filename) = tempfile.mkstemp()

        with open(filename, "wb") as f:
            f.write(content)

        conn = sqlite3.connect(filename)
        c = conn.cursor()
        try:
            c.execute(
                'select local_relpath, ".svn/pristine/" || substr(checksum,7,2) || "/" || substr(checksum,7) || ".svn-base" as alpha from NODES where kind="file";'
            )
            list_items = c.fetchall()
            # below functionality will find all usernames who have commited atleast once.
            c.execute("select distinct changed_author from nodes;")
            author_list = [r[0] for r in c.fetchall()]
            c.close()
        except Exception:
            raise FuzzExceptResourceParseError(
                "Error reading wc.db, either database corrupt or invalid file"
            )

        return author_list, list_items

    def process(self, fuzzresult):
        author_list, list_items = self.readwc(fuzzresult.history.content)

        if author_list:
            self.add_result("authors", "SVN authors", ", ".join(author_list))

        if list_items:
            for f, pristine in list_items:
                u = urljoin(fuzzresult.url.replace("/.svn/wc.db", "/"), f)
                if self.queue_url(u):
                    self.add_result("source", "SVN source code", f)



class ReqRespException(Exception):
    FATAL, RESOLVE_PROXY, RESOLVE_HOST, CONNECT_HOST, SSL, TIMEOUT = list(range(6))

    def __init__(self, etype, msg):
        self.etype = etype
        self.msg = msg
        Exception.__init__(self, msg)



import wx
import wx.py
import wx.grid
import wx.aui
import wx.html

import wx.lib.agw.aui as aui
import wx.dataview as dv
import wx.html2 as webview

if "2.8" in wx.version():
    import wx.lib.pubsub.setupkwargs
    from wx.lib.pubsub import pub
else:
    from wx.lib.pubsub import pub

try:
    from agw import pycollapsiblepane as PCP
except ImportError:  # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.pycollapsiblepane as PCP

# puedo no hacer paneles y devolverlos directamente como hace el treectrl, no se cual es la ventaja really

# ----------------------------------------------------------------------
# http://stackoverflow.com/questions/22265868/how-to-create-a-cmd-with-wxpython
# esto puede hacer q meta cmd.cmd directamente https://www.blog.pythonlibrary.org/2009/01/01/wxpython-redirecting-stdout-stderr/


class RedirectText(object):
    def __init__(self, aWxTextCtrl):
        self.out = aWxTextCtrl

    def write(self, string):
        self.out.WriteText(string)


class ConsolePanel(wx.Panel):
    def __init__(self, parent, interpreter):
        # begin wxGlade: MyFrame.__init__
        wx.Panel.__init__(self, parent, -1)

        self.history = []
        self.index = 0

        self.prompt = ">>"
        self.textctrl = wx.TextCtrl(
            self,
            -1,
            "",
            style=wx.TE_PROCESS_ENTER | wx.TE_MULTILINE | wx.TE_RICH,
            size=(-1, 250),
        )
        self.textctrl.SetForegroundColour(wx.WHITE)
        self.textctrl.SetBackgroundColour(wx.BLACK)

        self.textctrl.AppendText(self.prompt)

        self.textctrl.Bind(wx.EVT_CHAR, self.__bind_events)

        sizer = wx.BoxSizer()
        sizer.Add(self.textctrl, 1, wx.EXPAND)
        self.SetSizer(sizer)

        self._interp = interpreter
        redir = RedirectText(self.textctrl)

        import sys

        # Create a replacement for stdin.
        # self.reader = PseudoFileIn(self.readline, self.readlines)
        # self.reader.input = ''
        # self.reader.isreading = False

        # sys.stdin=self.reader
        sys.stdout = redir
        sys.stderr = redir

    def __bind_events(self, e):
        if e.GetKeyCode() == 13:
            self.index = len(self.history) - 1

            self.value = self.textctrl.GetValue()
            ln = self.get_last_line()

            ln = ln.strip()
            if ln not in self.history:
                self.history.append(ln)
            self.index += 1
            if ln:
                import shlex

                cmd = shlex.split(ln)
                # out en retvalue
                retvalue = self._interp.onecmd(cmd)
                if retvalue:
                    self.textctrl.WriteText("\n")
                    self.textctrl.AppendText(retvalue)

            self.textctrl.WriteText("\n")
            self.textctrl.WriteText(self.prompt)
        # down
        elif e.GetKeyCode() == 317:
            self.index += 1

            if self.index >= len(self.history):
                self.index = len(self.history) - 1

            self.textctrl.WriteText("\n")
            self.textctrl.WriteText(self.prompt)
            self.textctrl.WriteText(self.history[self.index])

        # up
        elif e.GetKeyCode() == 315:
            self.index -= 1

            if self.index < 0:
                self.index = 0

            self.textctrl.WriteText("\n")
            self.textctrl.WriteText(self.prompt)
            self.textctrl.WriteText(self.history[self.index])
        else:
            e.Skip()

    def get_last_line(self):
        nl = self.textctrl.GetNumberOfLines()
        ln = self.textctrl.GetLineText(nl - 1)
        ln = ln[len(self.prompt) :]

        return ln


# ----------------------------------------------------------------------


class ListPanel(wx.Panel):
    def __init__(self, parent, log, model, interpreter):
        self.log = log
        self._interp = interpreter
        wx.Panel.__init__(self, parent, -1)

        self.dvc = dv.DataViewCtrl(
            self, style=wx.BORDER_THEME | dv.DV_ROW_LINES | dv.DV_VERT_RULES
        )

        self.model = model
        self.dvc.AssociateModel(self.model)

        for row in list(self.model.row_mapper.values()):
            self.dvc.AppendTextColumn(row.title, row.colid, width=row.width)

        for c in self.dvc.Columns:
            c.Sortable = True
            c.Reorderable = True

        self.cp = cp = PCP.PyCollapsiblePane(
            self, label="Show console", agwStyle=wx.CP_GTK_EXPANDER
        )
        self.MakePaneContent(cp.GetPane())

        self.Sizer = wx.BoxSizer(wx.VERTICAL)
        self.Sizer.Add(self.dvc, 1, wx.EXPAND)
        self.Sizer.Add(cp, 0, wx.RIGHT | wx.LEFT | wx.EXPAND)
        self.SetSizer(self.Sizer)
        self.SetAutoLayout(True)

        self.dvc.Bind(dv.EVT_DATAVIEW_SELECTION_CHANGED, self.OnItemSelected)

    def OnItemSelected(self, event):
        try:
            items = self.dvc.GetSelections()
            item = self.model.GetRow(items[0])
        except IndexError:
            pass
        else:
            pub.sendMessage("selected_row", row=self.model.data[item])

        event.Skip()

    def MakePaneContent(self, pane):
        border = wx.BoxSizer()
        border.Add(ConsolePanel(pane, self._interp), wx.RIGHT | wx.LEFT | wx.EXPAND)
        # border.Add(py.shell.Shell(pane, InterpClass=self._interp, size=(-1,250)),  wx.RIGHT|wx.LEFT|wx.EXPAND)

        pane.SetSizer(border)


class HttpRawPanel(wx.Panel):
    def __init__(self, parent, frame):
        self._frame = frame
        wx.Panel.__init__(self, parent, -1)

        # self.req_txt = wx.TextCtrl(self, -1, "", style=wx.TE_MULTILINE|wx.TE_READONLY)
        self.req_txt = webview.WebView.New(self)
        # self.resp_txt = webview.WebView.New(self)
        self.resp_txt = wx.TextCtrl(
            self, -1, "", style=wx.TE_MULTILINE | wx.TE_READONLY
        )

        sizer = wx.BoxSizer(wx.HORIZONTAL)

        sizer.Add(self.req_txt, 1, wx.EXPAND)
        sizer.Add(self.resp_txt, 1, wx.EXPAND)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)

    def CreateHTMLCtrl(self):
        ctrl = wx.html.HtmlWindow(self, -1, wx.DefaultPosition, wx.Size(400, 300))
        if "gtk2" in wx.PlatformInfo or "gtk3" in wx.PlatformInfo:
            ctrl.SetStandardFonts()

        ctrl.SetPage("")

        return ctrl


class MainNotebookPanel(wx.Panel):
    def __init__(self, parent, frame, interpreter):
        self._frame = frame
        wx.Panel.__init__(self, parent, -1)

        bookStyle = aui.AUI_NB_DEFAULT_STYLE
        bookStyle &= ~(aui.AUI_NB_CLOSE_ON_ACTIVE_TAB)

        self.rawpanel = HttpRawPanel(self, self)
        self.renderpanel = self.create_web_view()

        self.nb = aui.AuiNotebook(self, style=bookStyle)
        self.nb.AddPage(self.rawpanel, "HTML Raw")
        self.nb.AddPage(self.renderpanel, "HTML Render")

        sizer = wx.BoxSizer()
        sizer.Add(self.nb, 1, wx.EXPAND)
        self.SetSizer(sizer)
        wx.CallAfter(self.nb.SendSizeEvent)

        pub.subscribe(self.on_selected_row, "selected_row")

    def create_web_view(self):
        return webview.WebView.New(self)

    def on_selected_row(self, row):
        from pygments import highlight
        from pygments.lexers import get_lexer_by_name
        from pygments.formatters import HtmlFormatter

        result = highlight(
            str(row.history), get_lexer_by_name("http"), HtmlFormatter(full=True)
        )
        # result2 = highlight(str(row.history.raw_content), get_lexer_by_name("http"), HtmlFormatter(full=True))

        self.renderpanel.SetPage(row.history.content, row.url)
        # self.rawpanel.req_txt.SetValue(str(row.history))
        self.rawpanel.req_txt.SetPage(result, "")
        # self.rawpanel.resp_txt.SetPage(result2, "")
        self.rawpanel.resp_txt.SetValue(str(row.history.raw_content))


# ----------------------------------------------------------------------


ID_About = wx.NewId()


class WfuzzFrame(wx.Frame):
    def __init__(
        self,
        parent,
        id=-1,
        title="Wfuzz",
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.DEFAULT_FRAME_STYLE | wx.SUNKEN_BORDER | wx.CLIP_CHILDREN,
    ):
        wx.Frame.__init__(self, parent, id, title, pos, size, style)

    def start_gui(self, controller):
        self.controller = controller
        # tell FrameManager to manage this frame
        self._mgr = wx.aui.AuiManager()
        self._mgr.SetManagedWindow(self)

        # create menu
        mb = wx.MenuBar()

        file_menu = wx.Menu()
        file_menu.Append(wx.ID_EXIT, "Exit")

        help_menu = wx.Menu()
        help_menu.Append(ID_About, "About...")

        mb.Append(file_menu, "File")
        mb.Append(help_menu, "Help")

        self.SetMenuBar(mb)

        self.SetMinSize(wx.Size(400, 300))

        # create some center panes
        self._mgr.AddPane(
            MainNotebookPanel(self, self, controller._interp),
            wx.aui.AuiPaneInfo()
            .Caption("Raw HTTP Content")
            .Name("analysis_notebook")
            .CenterPane(),
        )
        self._mgr.AddPane(
            self.CreateNotebook(),
            wx.aui.AuiPaneInfo().Name("main_notebook").CenterPane(),
        )
        self._mgr.Update()

        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.Bind(wx.EVT_MENU, self.OnExit, id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnAbout, id=ID_About)

        pub.subscribe(self.OnAddTab, "create_tab")

    def OnClose(self, event):
        pub.sendMessage("exit", msg="exiting...")
        self._mgr.UnInit()
        del self._mgr
        self.Destroy()

    def OnExit(self, event):
        pub.sendMessage("exit", msg="exiting...")
        self.Close()

    def OnAbout(self, event):
        msg = "WFuzz GUI\n(c) Copyright 2017, Xavi Mendez"
        dlg = wx.MessageDialog(self, msg, "About", wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def CreateNotebook(self):
        bookStyle = aui.AUI_NB_DEFAULT_STYLE
        # bookStyle &= ~(aui.AUI_NB_CLOSE_ON_ACTIVE_TAB)

        bookStyle = (
            aui.AUI_NB_DEFAULT_STYLE | aui.AUI_NB_TAB_EXTERNAL_MOVE | wx.NO_BORDER
        )

        client_size = self.GetClientSize()
        nb = aui.AuiNotebook(
            self,
            -1,
            wx.Point(client_size.x, client_size.y),
            wx.Size(430, 200),
            agwStyle=bookStyle,
        )

        nb.AddPage(
            ListPanel(self, self, self.controller._model, self.controller._interp),
            "Main",
        )

        return nb

    def OnAddTab(self, name, model, interp):
        auibook = self._mgr.GetPane("main_notebook").window

        auibook.AddPage(ListPanel(self, self, model, interp), name, True)

        self._mgr.Update()



import socket
import csv as csvmod
import json as jjson
from xml.dom import minidom

from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.plugin_api.base import BasePrinter
from wfuzz.exception import FuzzExceptPluginBadParams


@moduleman_plugin
class magictree(BasePrinter):
    name = "magictree"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "Prints results in magictree format"
    category = ["default"]
    priority = 99

    def __init__(self, output):
        BasePrinter.__init__(self, output)
        self.node_mt = None
        self.node_service = None

    def __create_xml_element(self, parent, caption, text):
        # Create a <xxx> element
        doc = minidom.Document()
        el = doc.createElement(caption)
        parent.appendChild(el)

        # Give the <xxx> element some text
        ptext = doc.createTextNode(text)

        el.appendChild(ptext)
        return el

    def header(self, summary):
        doc = minidom.Document()

        # <magictree class="MtBranchObject">
        self.node_mt = doc.createElement("magictree")
        self.node_mt.setAttribute("class", "MtBranchObject")

        # <testdata class="MtBranchObject">
        node_td = doc.createElement("testdata")
        node_td.setAttribute("class", "MtBranchObject")
        self.node_mt.appendChild(node_td)

        # <host>209.85.146.105
        host = summary.seed.history.host
        if host.find(":") > 0:
            host, port = host.split(":")
        else:
            port = 80
            if summary.seed.history.scheme.lower() == "https":
                port = 443

        try:
            resolving = socket.gethostbyname(host)
            node_h = self.__create_xml_element(node_td, "host", str(resolving))
        except socket.gaierror:
            node_h = self.__create_xml_element(node_td, "host", str(host))

        # <ipproto>tcp
        node_ipr = self.__create_xml_element(node_h, "ipproto", "tcp")

        # <port>80<state>open</state><service>http
        node_port = self.__create_xml_element(node_ipr, "port", str(port))
        self.__create_xml_element(node_port, "state", "open")
        if summary.seed.history.scheme.lower() == "https":
            node_port = self.__create_xml_element(node_port, "tunnel", "ssl")

        self.node_service = self.__create_xml_element(node_port, "service", "http")

    def result(self, fuzz_result):
        node_url = self.__create_xml_element(
            self.node_service, "url", str(fuzz_result.url)
        )

        if "Server" in fuzz_result.history.headers.response:
            self.__create_xml_element(
                node_url, "HTTPServer", fuzz_result.history.headers.response["Server"]
            )

        location = ""
        if "Location" in fuzz_result.history.headers.response:
            location = fuzz_result.history.headers.response["Location"]

        if fuzz_result.code == 301 or fuzz_result.code == 302 and location:
            self.__create_xml_element(node_url, "RedirectLocation", location)

        self.__create_xml_element(node_url, "ResponseCode", str(fuzz_result.code))
        self.__create_xml_element(node_url, "source", "WFuzz")

    def footer(self, summary):
        self.f.write(self.node_mt.toxml())


@moduleman_plugin
class html(BasePrinter):
    name = "html"
    author = (
        "Carlos del Ojo",
        "Christian Martorella",
        "Adapted to newer versions Xavi Mendez (@xmendez)",
    )
    version = "0.1"
    summary = "Prints results in html format"
    category = ["default"]
    priority = 99

    def __init__(self, output):
        BasePrinter.__init__(self, output)

    def header(self, summary):
        url = summary.url

        self.f.write(
            '<html><head></head><body bgcolor=#000000 text=#FFFFFF><h1>Fuzzing %s</h1>\r\n<table border="1">\r\n<tr><td>#request</td><td>Code</td><td>#lines</td><td>#words</td><td>Url</td></tr>\r\n'
            % (url)
        )

    def result(self, fuzz_result):
        htmlc = "<font>"

        if fuzz_result.code >= 400 and fuzz_result.code < 500:
            htmlc = "<font color=#FF0000>"
        elif fuzz_result.code >= 300 and fuzz_result.code < 400:
            htmlc = "<font color=#8888FF>"
        elif fuzz_result.code >= 200 and fuzz_result.code < 300:
            htmlc = "<font color=#00aa00>"

        if fuzz_result.history.method.lower() == "post":
            inputs = ""
            for n, v in list(fuzz_result.history.params.post.items()):
                inputs += '<input type="hidden" name="%s" value="%s">' % (n, v)

            self.f.write(
                '\r\n<tr><td>%05d</td>\r\n<td>%s%d</font></td>\r\n<td>%4dL</td>\r\n<td>%5dW</td>\r\n<td><table><tr><td>%s</td><td><form method="post" action="%s">%s<input type=submit name=b value="send POST"></form></td></tr></table></td>\r\n</tr>\r\n'
                % (
                    fuzz_result.nres,
                    htmlc,
                    fuzz_result.code,
                    fuzz_result.lines,
                    fuzz_result.words,
                    fuzz_result.description,
                    fuzz_result.url,
                    inputs,
                )
            )
        else:
            self.f.write(
                "\r\n<tr><td>%05d</td><td>%s%d</font></td><td>%4dL</td><td>%5dW</td><td><a href=%s>%s</a></td></tr>\r\n"
                % (
                    fuzz_result.nres,
                    htmlc,
                    fuzz_result.code,
                    fuzz_result.lines,
                    fuzz_result.words,
                    fuzz_result.url,
                    fuzz_result.url,
                )
            )

    def footer(self, summary):
        self.f.write("</table></body></html><h5>Wfuzz by EdgeSecurity<h5>\r\n")


@moduleman_plugin
class json(BasePrinter):
    name = "json"
    summary = "Results in json format"
    author = ("Federico (@misterade)", "Minor rework by Ilya Glotov (@ilyaglow)")
    version = "0.2"
    category = ["default"]
    priority = 99

    def __init__(self, output):
        BasePrinter.__init__(self, output)
        self.json_res = []

    def header(self, res):
        pass

    def result(self, res):
        server = ""
        if "Server" in res.history.headers.response:
            server = res.history.headers.response["Server"]
        location = ""
        if "Location" in res.history.headers.response:
            location = res.history.headers.response["Location"]
        elif res.history.url != res.history.redirect_url:
            location = "(*) %s" % res.history.url
        post_data = []
        if res.history.method.lower() == "post":
            for n, v in list(res.history.params.post.items()):
                post_data.append({"parameter": n, "value": v})

        res_entry = {
            "chars": res.chars,
            "code": res.code,
            "payload": res.description,
            "lines": res.lines,
            "location": location,
            "method": res.history.method,
            "post_data": post_data,
            "server": server,
            "url": res.url,
            "words": res.words,
        }
        self.json_res.append(res_entry)

    def footer(self, summary):
        self.f.write(jjson.dumps(self.json_res))


@moduleman_plugin
class raw(BasePrinter):
    name = "raw"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "Raw output format"
    category = ["default"]
    priority = 99

    def __init__(self, output):
        BasePrinter.__init__(self, output)

    def header(self, summary):
        self.f.write("Target: %s\n" % summary.url)

        if summary.total_req > 0:
            self.f.write("Total requests: %d\n" % summary.total_req)
        else:
            self.f.write("Total requests: <<unknown>>\n")

        if self.verbose:
            self.f.write(
                "==============================================================================================================================================\n"
            )
            self.f.write(
                "ID    C.Time   Response   Lines      Word         Chars                  Server                                             Redirect   Payload    \n"
            )
            self.f.write(
                "==============================================================================================================================================\n"
            )
        else:
            self.f.write(
                "==================================================================\n"
            )
            self.f.write(
                "ID    Response   Lines      Word         Chars          Request    \n"
            )
            self.f.write(
                "==================================================================\n"
            )

    def _print_verbose(self, res):
        self.f.write("%05d:  " % res.nres)
        self.f.write("%.3fs   C=" % res.timer)

        location = ""
        if "Location" in res.history.headers.response:
            location = res.history.headers.response["Location"]
        elif res.history.url != res.history.redirect_url:
            location = "(*) %s" % res.history.url

        server = ""
        if "Server" in res.history.headers.response:
            server = res.history.headers.response["Server"]

        if res.exception:
            self.f.write("XXX")
        else:
            self.f.write("%05d:  C=%03d" % (res.nres, res.code))

        self.f.write(
            '   %4d L\t   %5d W\t  %5d Ch  %20.20s  %51.51s   "%s"\n'
            % (
                res.lines,
                res.words,
                res.chars,
                server[:17],
                location[:48],
                res.description,
            )
        )

        for plugin_res in res.plugins_res:
            if plugin_res.is_visible(self.verbose):
                self.f.write(
                    " |_  {} {}\n".format(
                        plugin_res.issue, plugin_res.data if plugin_res.data else ""
                    )
                )

    def _print(self, res):
        if res.exception:
            self.f.write("XXX")
        else:
            self.f.write("%05d:  C=%03d" % (res.nres, res.code))

        self.f.write(
            '   %4d L\t   %5d W\t  %5d Ch\t  "%s"\n'
            % (res.lines, res.words, res.chars, res.description)
        )

        for plugin_res in res.plugins_res:
            if plugin_res.is_visible(self.verbose):
                self.f.write(
                    " |_  {} {}\n".format(
                        plugin_res.issue, plugin_res.data if plugin_res.data else ""
                    )
                )

    def result(self, res):
        if self.verbose:
            self._print_verbose(res)
        else:
            self._print(res)

    def footer(self, summary):
        self.f.write("\n")
        self.f.write("Total time: %s\n" % str(summary.totaltime)[:8])

        if summary.backfeed() > 0:
            self.f.write(
                "Processed Requests: %s (%d + %d)\n"
                % (
                    str(summary.processed())[:8],
                    (summary.processed() - summary.backfeed()),
                    summary.backfeed(),
                )
            )
        else:
            self.f.write("Processed Requests: %s\n" % (str(summary.processed())[:8]))
        self.f.write("Filtered Requests: %s\n" % (str(summary.filtered())[:8]))
        self.f.write(
            "Requests/sec.: %s\n"
            % str(
                summary.processed() / summary.totaltime if summary.totaltime > 0 else 0
            )[:8]
        )


@moduleman_plugin
class field(BasePrinter):
    name = "field"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "Raw output format only showing the specified field expression. No header or footer."
    category = ["default"]
    priority = 99

    def __init__(self, output):
        BasePrinter.__init__(self, output)

    def header(self, summary):
        pass

    def result(self, res):
        if res._fields:
            to_print = res._field("\n")
            if to_print:
                print(to_print)
        else:
            raise FuzzExceptPluginBadParams(
                "You need to supply  valid --field or --efield expression for unsing this printer."
            )

    def footer(self, summary):
        pass


@moduleman_plugin
class csv(BasePrinter):
    name = "csv"
    author = (
        "@Yoginski initial version",
        "Adapted by @egilas to work in newer version of wfuzz",
    )
    summary = "CSV printer ftw"
    version = "1.0"
    category = ["default"]
    priority = 99

    def write(self, e):
        self.f.write(e)
        pass

    def __init__(self, output):
        BasePrinter.__init__(self, output)
        self.csv_writer = csvmod.writer(self)

    def header(self, summary):
        self._print_csv(
            ["id", "response", "lines", "word", "chars", "request", "success"]
        )

    def result(self, res):
        line = [
            res.nres,
            res.code,
            res.lines,
            res.words,
            res.chars,
            res.description,
            0 if res.exception else 1,
        ]
        self._print_csv(line)

    def noresult(self, res):
        pass

    def footer(self, summary):
        pass

    def _print_csv(self, values):
        self.csv_writer.writerow(values)



from collections import namedtuple
import wx.dataview as dv

from wfuzz.filters.ppfilter import FuzzResFilter

Row = namedtuple("Row", "title colid width rtype field")


class GUIModel(dv.PyDataViewIndexListModel):
    def __init__(self, data=None):
        self.data = data if data is not None else []
        dv.PyDataViewIndexListModel.__init__(self, len(self.data))

        self.row_mapper = {
            0: Row(title="ID", colid=0, width=100, rtype="int", field="nres"),
            1: Row(title="Response", colid=1, width=100, rtype="int", field="code"),
            2: Row(title="Lines", colid=2, width=170, rtype="int", field="lines"),
            3: Row(title="Words", colid=3, width=170, rtype="int", field="words"),
            4: Row(title="Chars", colid=4, width=170, rtype="int", field="chars"),
            5: Row(
                title="Payload", colid=5, width=170, rtype="string", field="description"
            ),
        }

    def GetColumnType(self, col):
        return self.row_mapper[col].rtype

    def GetValueByRow(self, row, col):
        try:
            return self.data[row].get_field(self.row_mapper[col].field)
        except IndexError:
            return ""

    def GetColumnCount(self):
        return len(self.row_mapper)

    def GetCount(self):
        return len(self.data)

    # Called to check if non-standard attributes should be used in the
    # cell at (row, col)
    def GetAttrByRow(self, row, col, attr):
        # #self.log.write('GetAttrByRow: (%d, %d)' % (row, col))
        # if col == 3:
        #     attr.SetColour('blue')
        #     attr.SetBold(True)
        #     return True
        return False

    def Compare(self, item1, item2, col, ascending):
        if not ascending:  # swap sort order?
            item2, item1 = item1, item2
        row1 = self.GetRow(item1)
        row2 = self.GetRow(item2)

        value1 = self.GetValueByRow(row1, col)
        value2 = self.GetValueByRow(row2, col)
        if self.row_mapper[col].rtype == "int":
            value1 = int(value1)
            value2 = int(value2)

        return (value1 > value2) - (value1 < value2)

    def DeleteRows(self, rows):
        # make a copy since we'll be sorting(mutating) the list
        rows = list(rows)
        # use reverse order so the indexes don't change as we remove items
        rows.sort(reverse=True)

        for row in rows:
            # remove it from our data structure
            del self.data[row]
            # notify the view(s) using this model that it has been removed
            self.RowDeleted(row)

    def AddRow(self, value):
        self.data.append(value)
        self.RowAppended()

    def Clear(self):
        self.data = []
        self.Cleared()

    def DeleteRows_by_filter(self, filter_string):
        ffilter = FuzzResFilter(filter_string=filter_string)

        for row, item in reversed(list(enumerate(self.data))):
            if ffilter.is_visible(item):
                del self.data[row]
                self.RowDeleted(row)



from .TextParser import TextParser
import json


class Variable:
    def __init__(self, name, value="", extraInfo=""):
        self.name = name
        self.value = value
        self.initValue = value
        self.extraInfo = extraInfo

    def restore(self):
        self.value = self.initValue

    def change(self, newval):
        self.initValue = self.value = newval

    def update(self, val):
        self.value = val

    def append(self, val):
        self.value += val

    def __str__(self):
        return "[ %s : %s ]" % (self.name, self.value)


class VariablesSet:
    def __init__(self):
        self.variables = []
        self.boundary = None

    def names(self):
        dicc = []
        for i in self.variables:
            dicc.append(i.name)

        return dicc

    def existsVar(self, name):
        return name in self.names()

    def addVariable(self, name, value="", extraInfo=""):
        self.variables.append(Variable(name, value, extraInfo))

    def getVariable(self, name):
        dicc = []
        for i in self.variables:
            if i.name == name:
                dicc.append(i)

        if len(dicc) > 1:
            raise Exception("Variable exists more than one time!!! :D" % (name))

        if not dicc:
            var = Variable(name)
            self.variables.append(var)
            return var

        return dicc[0]

    def urlEncoded(self):
        return "&".join(
            [
                "=".join([i.name, i.value]) if i.value is not None else i.name
                for i in self.variables
            ]
        )

    def json_encoded(self):
        dicc = {i.name: i.value for i in self.variables}

        return json.dumps(dicc)

    def parse_json_encoded(self, cad):
        dicc = []

        for key, value in json.loads(cad).items():
            dicc.append(Variable(key, value))

        self.variables = dicc

    def parseUrlEncoded(self, cad):
        dicc = []

        if cad == "":
            dicc.append(Variable("", None))

        for i in cad.split("&"):
            if i:
                var_list = i.split("=", 1)
                if len(var_list) == 1:
                    dicc.append(Variable(var_list[0], None))
                elif len(var_list) == 2:
                    dicc.append(Variable(var_list[0], var_list[1]))

        self.variables = dicc

    def multipartEncoded(self):
        if not self.boundary:
            self.boundary = "---------------------------D33PB1T0R3QR3SP0B0UND4RY2203"
        pd = ""
        for i in self.variables:
            pd += "--" + self.boundary + "\r\n"
            pd += "%s\r\n\r\n%s\r\n" % ("\r\n".join(i.extraInfo), i.value)
        pd += "--" + self.boundary + "--\r\n"
        return pd

    def parseMultipart(self, cad, boundary):
        self.boundary = boundary
        dicc = []
        tp = TextParser()
        tp.setSource("string", cad)

        while True:
            headers = []
            if not tp.readUntil('name="([^"]+)"'):
                break
            var = tp[0][0]
            headers.append(tp.lastFull_line.strip())
            while True:
                tp.readLine()
                if tp.search("^([^:]+): (.*)$"):
                    headers.append(tp.lastFull_line.strip())
                else:
                    break

            value = ""
            while True:
                tp.readLine()
                if not tp.search(boundary):
                    value += tp.lastFull_line
                else:
                    break

            if value[-2:] == "\r\n":
                value = value[:-2]

            dicc.append(Variable(var, value.strip(), headers))

        self.variables = dicc



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.exception import FuzzExceptPluginBadParams
from wfuzz.plugin_api.base import BasePayload
from wfuzz.fuzzobjects import FuzzWordType


@moduleman_plugin
class range(BasePayload):
    name = "range"
    author = (
        "Carlos del Ojo",
        "Christian Martorella",
        "Adapted to newer versions Xavi Mendez (@xmendez)",
    )
    version = "0.1"
    description = ("ie. 0-10",)
    summary = "Returns each number of the given range."
    category = ["default"]
    priority = 99

    parameters = (("range", "", True, "Range of numbers in the form 0-10."),)

    default_parameter = "range"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        try:
            ran = self.params["range"].split("-")
            self.minimum = int(ran[0])
            self.maximum = int(ran[1])
            self.__count = self.maximum - self.minimum + 1
            self.width = len(ran[0])
            self.current = self.minimum
        except ValueError:
            raise FuzzExceptPluginBadParams('Bad range format (eg. "23-56")')

    def get_type(self):
        return FuzzWordType.WORD

    def get_next(self):
        if self.current > self.maximum:
            raise StopIteration
        else:
            if self.width:
                payl = "%0" + str(self.width) + "d"
                payl = payl % (self.current)
            else:
                payl = str(self.current)

            self.current += 1
            return payl

    def count(self):
        return self.__count

    def __iter__(self):
        return self



# http://code.activestate.com/recipes/134892/
#
# getch()-like unbuffered character reading from stdin on both Windows and Unix (Python recipe)
# A small utility class to read single characters from standard input, on both Windows and UNIX systems. It provides a getch() function-like instance.
#
# Created by Danny Yoo on Fri, 21 Jun 2002 (PSF)
# (http://code.activestate.com/recipes/users/98032/)


class _Getch:
    """Gets a single character from standard input.  Does not echo to the screen."""

    def __init__(self):
        try:
            self.impl = _GetchWindows()
        except ImportError:
            try:
                self.impl = _GetchUnix()
            except ImportError:
                self.impl = _GetchMacCarbon()

    def __call__(self):
        return self.impl()


class _GetchUnix:
    def __init__(self):
        import tty
        import sys

    def __call__(self):
        import sys
        import tty
        import termios

        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch


class _GetchWindows:
    def __init__(self):
        import msvcrt

    def __call__(self):
        import msvcrt


class _GetchMacCarbon:
    """
    A function which returns the current ASCII key that is down;
    if no ASCII key is down, the null string is returned.  The
    page http://www.mactech.com/macintosh-c/chap02-1.html was
    very helpful in figuring out how to do this.
    """

    def __init__(self):
        import Carbon

        # see if it has this (in Unix, it doesn't)
        Carbon.Evt

    def __call__(self):
        import Carbon

        if Carbon.Evt.EventAvail(0x0008)[0] == 0:  # 0x0008 is the keyDownMask
            return ""
        else:
            #
            # The event contains the following info:
            # (what,msg,when,where,mod)=Carbon.Evt.GetNextEvent(0x0008)[1]
            #
            # The message (msg) contains the ASCII char which is
            # extracted with the 0x000000FF charCodeMask; this
            # number is converted to an ASCII character with chr() and
            # returned
            #
            (what, msg, when, where, mod) = Carbon.Evt.GetNextEvent(0x0008)[1]
            return chr(msg & 0x000000FF)


if __name__ == "__main__":
    print("Press a key")
    inkey = _Getch()

    import sys

    for i in range(sys.maxsize):
        k = inkey()
        if k != "":
            break
    print("you pressed ", k)



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.plugin_api.base import BasePayload
from wfuzz.exception import FuzzExceptBadOptions
from wfuzz.fuzzobjects import FuzzWordType


@moduleman_plugin
class permutation(BasePayload):
    name = "permutation"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    description = ()
    summary = "Returns permutations of the given charset and length."
    category = ["default"]
    priority = 99

    parameters = (("ch", "", True, "Charset and len to permute in the form of abc-2."),)

    default_parameter = "ch"

    def __init__(self, params):
        BasePayload.__init__(self, params)
        self.charset = []

        try:
            ran = self.params["ch"].split("-")
            self.charset = ran[0]
            self.width = int(ran[1])
        except ValueError:
            raise FuzzExceptBadOptions('Bad range format (eg. "0-ffa")')

        pset = []
        for x in self.charset:
            pset.append(x)

        words = self.xcombinations(pset, self.width)
        self.lista = []
        for x in words:
            self.lista.append("".join(x))

        self.__count = len(self.lista)

    def count(self):
        return self.__count

    def get_type(self):
        return FuzzWordType.WORD

    def get_next(self):
        if self.lista != []:
            payl = self.lista.pop()
            return payl
        else:
            raise StopIteration

    def xcombinations(self, items, n):
        if n == 0:
            yield []
        else:
            for i in range(len(items)):
                for cc in self.xcombinations(items[:i] + items[i:], n - 1):
                    yield [items[i]] + cc



from wfuzz.plugin_api.base import BasePlugin
from wfuzz.externals.moduleman.plugin import moduleman_plugin


KBASE_NEW_COOKIE = "cookies.cookie"


@moduleman_plugin
class cookies(BasePlugin):
    name = "cookies"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "Looks for new cookies"
    description = ("Looks for new cookies",)
    category = ["info", "passive", "default"]
    priority = 99

    parameters = ()

    def __init__(self):
        BasePlugin.__init__(self)

    def validate(self, fuzzresult):
        return True

    def process(self, fuzzresult):
        new_cookies = list(fuzzresult.history.cookies.response.items())

        if len(new_cookies) > 0:
            for name, value in new_cookies:

                if (
                    name != ""
                    and KBASE_NEW_COOKIE not in self.kbase
                    or name not in self.kbase[KBASE_NEW_COOKIE]
                ):
                    self.kbase[KBASE_NEW_COOKIE] = name
                    self.add_result(
                        "cookie", "Cookie first set", "%s=%s" % (name, value)
                    )



from wfuzz.exception import (
    FuzzExceptMissingAPIKey,
    FuzzExceptResourceParseError,
    FuzzExceptPluginLoadError,
)
from wfuzz.facade import Facade
from wfuzz.helpers.utils import MyCounter


# Python 2 and 3: alternative 4
try:
    from urllib.request import Request
    from urllib.request import build_opener
except ImportError:
    from urllib2 import Request
    from urllib2 import build_opener

import json

# python 2 and 3: iterator
from builtins import object
from threading import Thread
from queue import Queue

IMPORTED_SHODAN = True
try:
    import shodan
except ImportError:
    IMPORTED_SHODAN = False

m = {
    "matches": [
        {
            "_shodan": {
                "id": "54e0ae62-9e22-404b-91b4-92f99e89c987",
                "options": {},
                "ptr": True,
                "module": "auto",
                "crawler": "62861a86c4e4b71dceed5113ce9593b98431f89a",
            },
            "hash": -1355923443,
            "os": None,
            "ip": 1240853908,
            "isp": "Comcast Cable",
            "http": {
                "html_hash": -2142469325,
                "robots_hash": None,
                "redirects": [],
                "securitytxt": None,
                "title": "400 Bad Request",
                "sitemap_hash": None,
                "robots": None,
                "favicon": None,
                "host": "73.245.237.148",
                "html": '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">\n<html><head>\n<title>400 Bad Request</title>\n</head><body>\n<h1>Bad Request</h1>\n<p>Your browser sent a request that this server could not understand.<br />\nReason: You\'re speaking plain HTTP to an SSL-enabled server port.<br />\n Instead use the HTTPS scheme to access this URL, please.<br />\n</p>\n<p>Additionally, a 404 Not Found\nerror was encountered while trying to use an ErrorDocument to handle the request.</p>\n</body></html>\n',
                "location": "/",
                "components": {},
                "server": "Apache",
                "sitemap": None,
                "securitytxt_hash": None,
            },
            "port": 9445,
            "hostnames": ["c-73-245-237-148.hsd1.fl.comcast.net"],
            "location": {
                "city": "Fort Lauderdale",
                "region_code": "FL",
                "area_code": 954,
                "longitude": -80.3704,
                "country_code3": "USA",
                "country_name": "United States",
                "postal_code": "33331",
                "dma_code": 528,
                "country_code": "US",
                "latitude": 26.065200000000004,
            },
            "timestamp": "2019-04-10T10:30:48.297701",
            "domains": ["comcast.net"],
            "org": "Comcast Cable",
            "data": "HTTP/1.1 400 Bad Request\r\nDate: Wed, 10 Apr 2019 10:19:07 GMT\r\nServer: Apache\r\nContent-Length: 481\r\nConnection: close\r\nContent-Type: text/html; charset=iso-8859-1\r\n\r\n",
            "asn": "AS7922",
            "transport": "tcp",
            "ip_str": "73.245.237.148",
        },
        {
            "_shodan": {
                "id": "4ace6fd1-8295-4aea-a086-2280598ca9e7",
                "options": {},
                "ptr": True,
                "module": "auto",
                "crawler": "62861a86c4e4b71dceed5113ce9593b98431f89a",
            },
            "product": "Apache httpd",
            "hash": 370611044,
            "os": None,
            "ip": 35226500,
            "isp": "EE High Speed Internet",
            "http": {
                "html_$ ash": -163723763,
                "robots_hash": None,
                "redirects": [],
                "securitytxt": None,
                "title": "401 Authorization Required",
                "sitemap_hash": None,
                "robots": None,
                "favicon": None,
                "host": "2.25.131.132",
                "html": "<HEAD><TITLE>401 Authorization Required</TITLE></HEAD>\n<BODY><H1>401 Authoriza$ ion Required</H1>\nBrowser not authentication-capable or authentication failed.\n</BODY>\n",
                "location": "/",
                "components": {},
                "server": "Apache",
                "sitemap": None,
                "securitytxt_hash": None,
            },
            "cpe": ["cpe:/a:apache:http_server"],
            "port": 8085,
            "hostnames": [],
            "location": {
                "city": "$ helmsford",
                "region_code": "E4",
                "area_code": None,
                "longitude": 0.48330000000001405,
                "country_code3": "GBR",
                "country_name": "United Kingdom",
                "postal_code": "CM2",
                "dma_code": None,
                "country_code": "GB",
                "latitude": 51.733300000000014,
            },
            "timestamp": "2019-04-10T11:03:59.955967",
            "$ omains": [],
            "org": "EE High Speed Internet",
            "data": 'HTTP/1.1 401 Unauthorized\r\nServer: Apache\r\nConnection: Close\r\nContent-type: text/html\r\nWWW-Authenticate: Digest realm="DSLForum CPE Management", algorithm=MD5, qop=auth, stale=FALSE, nonce="3d7a3f71e72e095dba31fd77d4db74$5", opaque="5ccc069c403ebaf9f0171e9517f40e41"\r\n\r\n',
            "asn": "AS12576",
            "transport": "tcp",
            "ip_str": "2.25.131.132",
        },
    ]
}


class BingIter(object):
    def __init__(self, dork, offset=0, limit=0, key=None):
        if key is None:
            key = Facade().sett.get("plugins", "bing_apikey")

        if not key:
            raise FuzzExceptMissingAPIKey(
                "An api Bing key is needed. Please chek wfuzz.ini."
            )

        self._key = key
        self._dork = dork

        self.max_count = 0
        self.current = 0
        self._index = 0
        self._retrieved = 0
        self._results = []

        # first bing request to get estimated total count (it does not take into consideration offset).
        if limit > 0 and limit < 50:
            total_results, self._retrieved, self._results = self._do_search(
                offset, limit
            )
        else:
            total_results, self._retrieved, self._results = self._do_search(offset)

        # offset not over the results
        if offset > total_results:
            self._offset = total_results
        else:
            self._offset = offset

        self.max_count = total_results - self._offset

        # no more than limit results
        if self.max_count > limit and limit > 0:
            self.max_count = limit

    def _do_search(self, offset=0, limit=50):
        # some code taken from http://www.securitybydefault.com/2014/07/search2auditpy-deja-que-bing-haga-el.html?utm_source=feedburner&utm_medium=feed&utm_campaign=Feed%3A+SecurityByDefault+%28Security+By+Default%29
        # api doc http://go.microsoft.com/fwlink/?LinkID=248077
        user_agent = "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; FDM; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 1.1.4322)"
        creds = (":%s" % self._key).encode("base64")[:-1]
        auth = "Basic %s" % creds

        result = None

        try:
            urlstr = (
                "https://api.datamarket.azure.com/Data.ashx/Bing/Search/Composite?Sources=%27web%27&Query=%27"
                + self._dork
                + "%27&$format=json"
            )
            if limit != 50:
                urlstr += "&$top=%d" % limit
            if offset != 0:
                urlstr += "&$skip=%d" % offset

            request = Request(urlstr)

            request.add_header("Authorization", auth)
            request.add_header("User-Agent", user_agent)
            requestor = build_opener()
            result = requestor.open(request)
        except Exception as e:
            raise FuzzExceptResourceParseError(
                "Error when retrieving Bing API results: %s." % str(e)
            )

        results = json.loads(result.read())

        # WebTotal is not reliable, it is usually much bigger than the actual results, therefore
        # if your offset increases over the real number of results, you get a dict
        # without values and counters to ''. It gets updated when you are close to that limit though.
        if results["d"]["results"][0]["WebTotal"]:
            res_total = int(results["d"]["results"][0]["WebTotal"])
            res_list = results["d"]["results"][0]["Web"]

            return res_total, len(res_list), res_list
        else:
            return 0, 0, 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.current >= self.max_count:
            raise StopIteration

        # Result buffer already consumed
        if self._index >= self._retrieved:
            realcount, self._retrieved, self._results = self._do_search(
                self.current + self._offset
            )

            self._index = 0

            # update real count
            if self.max_count > realcount:
                self.max_count = realcount

        elem = self._results[self._index]["Url"].strip()

        self.current += 1
        self._index += 1

        # pycurl does not like unicode
        if isinstance(elem, str):
            return elem.encode("utf-8")
        else:
            return elem


class ShodanIter:
    SHODAN_RES_PER_PAGE = 100
    MAX_ENQUEUED_RES = SHODAN_RES_PER_PAGE + 1
    NUM_OF_WORKERS = 1
    SLOW_START = True

    def __init__(self, dork, page, limit):
        if IMPORTED_SHODAN is False:
            raise FuzzExceptPluginLoadError(
                "shodan module not imported. Please, install shodan using pip"
            )

        key = Facade().sett.get("plugins", "shodan_apikey")
        if not key:
            raise FuzzExceptMissingAPIKey(
                "A Shodan api key is needed. Please check ~/.wfuzz/wfuzz.ini"
            )

        self.api = shodan.Shodan(key)
        self._dork = dork
        self._page = MyCounter(page)
        self._page_limit = self._page() + limit if limit > 0 else -1

        self.results_queue = Queue(self.MAX_ENQUEUED_RES)
        self.page_queue = Queue()

        self._threads = []

        self._started = False
        self._cancel_job = False

    def _do_search(self):
        while 1:
            page = self.page_queue.get()
            if page is None:
                self.page_queue.task_done()
                break

            if self._cancel_job:
                self.page_queue.task_done()
                continue

            if self._page_limit > 0 and page >= self._page_limit:
                self.page_queue.task_done()
                self.results_queue.put(None)
                continue

            try:
                results = self.api.search(self._dork, page=page)
                for item in results["matches"]:
                    if not self._cancel_job:
                        self.results_queue.put(item)

                self.page_queue.task_done()
                if not self._cancel_job:
                    self.page_queue.put(self._page.inc())
            except shodan.APIError as e:
                self.page_queue.task_done()
                if "Invalid page size" in str(e):
                    self.results_queue.put(None)
                elif "Insufficient query credits" in str(e):
                    self.results_queue.put(None)
                else:
                    self.results_queue.put(e)
                continue

    def __iter__(self):
        return self

    def _start(self):
        for th_n in range(self.NUM_OF_WORKERS):
            worker = Thread(target=self._do_search)
            worker.setName("_do_search_{}".format(str(th_n)))
            self._threads.append(worker)
            worker.start()

        self.page_queue.put(self._page())
        if not self.SLOW_START:
            for _ in range(self.NUM_OF_WORKERS - 1):
                self.page_queue.put(self._page.inc())

    def _stop(self):
        self._cancel_job = True

        for th in self._threads:
            self.page_queue.put(None)

        self.page_queue.join()

        for th in self._threads:
            th.join()

        self._threads = []

        self.results_queue.put(None)

    def __next__(self):
        if not self._started:
            self._start()
            self._started = True

        res = self.results_queue.get()
        self.results_queue.task_done()

        if res is None:
            self._stop()
            self._cancel_job = False
            self._started = False
            raise StopIteration
        elif isinstance(res, Exception):
            self._stop()
            raise res

        return res



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.exception import FuzzExceptBadFile
from wfuzz.plugin_api.base import BasePayload
from wfuzz.helpers.file_func import FileDetOpener
from wfuzz.fuzzobjects import FuzzWordType


@moduleman_plugin
class file(BasePayload):
    name = "file"
    author = (
        "Carlos del Ojo",
        "Christian Martorella",
        "Adapted to newer versions Xavi Mendez (@xmendez)",
    )
    version = "0.2"
    description = ("Returns the contents of a dictionary file line by line.",)
    summary = "Returns each word from a file."
    category = ["default"]
    priority = 99

    parameters = (
        ("fn", "", True, "Filename of a valid dictionary"),
        (
            "count",
            "True",
            False,
            "Indicates if the number of words in the file should be counted.",
        ),
        ("encoding", "Auto", False, "Indicates the file encoding."),
    )

    default_parameter = "fn"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        try:
            encoding = (
                self.params["encoding"]
                if self.params["encoding"].lower() != "auto"
                else None
            )
            self.f = FileDetOpener(self.find_file(self.params["fn"]), encoding)
        except IOError as e:
            raise FuzzExceptBadFile("Error opening file. %s" % str(e))

        self.__count = None

    def get_type(self):
        return FuzzWordType.WORD

    def get_next(self):
        line = next(self.f)
        if not line:
            self.f.close()
            raise StopIteration
        return line.strip()

    def count(self):
        if self.params["count"].lower() == "false":
            return -1

        if self.__count is None:
            self.__count = len(list(self.f))
            self.f.reset()

        return self.__count



import inspect
import logging
import importlib.util
import os.path


class IModuleLoader:
    def __init__(self, **params):
        self.set_params(**params)

    def set_params(self, **params):
        raise NotImplementedError

    def load(self, registrant):
        raise NotImplementedError


class FileLoader(IModuleLoader):
    def __init__(self, **params):
        IModuleLoader.__init__(self, **params)
        self.__logger = logging.getLogger("libraries.FileLoader")

    def set_params(self, **params):
        if "base_path" not in params:
            return
        elif "filename" not in params:
            return

        self.filename = params["filename"]
        self.base_path = params["base_path"]
        if self.base_path.endswith("/"):
            self.base_path = self.base_path[:-1]

    def load(self, registrant):
        self.module_registrant = registrant

        self._load_py_from_file(os.path.join(self.base_path, self.filename))

    def _build_id(self, filename, objname):
        filepath, filename = os.path.split(filename)

        relative_path = os.path.relpath(filepath, self.base_path)
        identifier = relative_path + "/" + objname
        if identifier.startswith("./"):
            identifier = identifier[2:]

        return identifier

    def _load_py_from_file(self, filename):
        """
        Opens "filename", inspects it and calls the registrant
        """
        self.__logger.debug("__load_py_from_file. START, file=%s" % (filename,))

        module_name = os.path.splitext(os.path.basename(filename))[0]

        try:
            spec = importlib.util.spec_from_file_location(module_name, filename)
            if spec and spec.loader:
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)
            else:
                raise ImportError(f"Could not load spec for {filename}")
        except ImportError as msg:
            self.__logger.critical(
                "__load_py_from_file. Filename: %s Exception, msg=%s" % (filename, msg)
            )
            return
        except SyntaxError as msg:
            self.__logger.critical(
                "__load_py_from_file. Filename: %s Exception, msg=%s" % (filename, msg)
            )
            return

        for objname in dir(module):
            obj = getattr(module, objname)
            self.__logger.debug("__load_py_from_file. inspecting=%s" % (objname,))
            if inspect.isclass(obj):
                if "__PLUGIN_MODULEMAN_MARK" in dir(obj):
                    if self.module_registrant:
                        self.module_registrant.register(
                            self._build_id(filename, objname), obj
                        )

        self.__logger.debug("__load_py_from_file. END, loaded file=%s" % (filename,))


class DirLoader(FileLoader):
    def __init__(self, **params):
        FileLoader.__init__(self, **params)
        self.__logger = logging.getLogger("libraries.DirLoader")

    def set_params(self, **params):
        if "base_dir" not in params:
            return
        elif "base_path" not in params:
            return

        self.base_dir = params["base_dir"]
        self.base_path = params["base_path"]
        if self.base_path.endswith("/"):
            self.base_path = self.base_path[:-1]

    def load(self, registrant):
        self.module_registrant = registrant
        self.structure = self.__load_all(self.base_dir)

    def _build_id(self, filename, objname):
        filepath, filename = os.path.split(filename)

        relative_path = os.path.relpath(
            filepath, os.path.join(self.base_path, self.base_dir)
        )
        identifier = relative_path + "/" + objname
        if identifier.startswith("./"):
            identifier = identifier[2:]

        return identifier

    def __load_all(self, dir_name):
        """
        loads all plugins and creates a loaded list of scripts from directory plugins like:
        [ ( category,[script1, script2,...] ), (category2,[script1, (subcategory,[script1,script2]),...]) ]
        """
        walked = []

        current = os.path.join(self.base_path, dir_name)
        if os.path.isdir(current):
            dir_list = self.__walk_dir_tree(current)
            walked.append((current, dir_list))
            if self.module_registrant:
                self.module_registrant.end_loading()

        return walked

    def __walk_dir_tree(self, dirname):
        dir_list = []

        self.__logger.debug("__walk_dir_tree. START dir=%s", dirname)

        for f in os.listdir(dirname):
            current = os.path.join(dirname, f)
            if os.path.isfile(current) and f.endswith("py"):
                if self.module_registrant:
                    self._load_py_from_file(current)

                dir_list.append(current)
            elif os.path.isdir(current):
                ret = self.__walk_dir_tree(current)
                if ret:
                    dir_list.append((f, ret))

        return dir_list



import wx

from .ui.console.clparser import CLParser
from .ui.gui.model import GUIModel
from .facade import Facade

if "2.8" in wx.version():
    import wx.lib.pubsub.setupkwargs
    from wx.lib.pubsub import pub
else:
    from wx.lib.pubsub import pub


class WfuzzInterpreter:
    def __init__(self, model):
        self.model = model

    def onecmd(self, cmd):
        if cmd[0] == "wfuzz":
            self.do_wfuzz(cmd)
        elif cmd[0] == "clear":
            self.model.Clear()
        elif cmd[0] == "wfilter":
            self.do_wfilter(cmd)
        elif cmd[0] == "newtab":
            self.do_tab(cmd)
        elif cmd[0] == "del":
            self.do_delete(cmd)

    def do_wfilter(self, cmd):
        from wfuzz.core import dictionary

        try:
            session_options = CLParser(cmd).parse_cl()
        except SystemExit:
            print("\n")
            pass
        except Exception:
            pass
        else:
            for res in dictionary.from_options(session_options):
                r = res[0]
                if "FuzzResult" in str(r.__class__):
                    r._fields = r.url

                self.model.AddRow(r)

    def do_wfuzz(self, cmd):
        try:
            session_options = CLParser(cmd).parse_cl().compile()
        except SystemExit:
            print("\n")
            pass
        except Exception:
            pass
        else:
            for res in session_options.fuzz():
                self.model.AddRow(res)

    def do_delete(self, cmd):
        self.model.DeleteRows_by_filter(cmd[1])

    def do_tab(self, cmd):
        data = Facade().data[cmd[1]] = []
        model = GUIModel(data)
        pub.sendMessage(
            "create_tab", name=cmd[1], model=model, interp=WfuzzInterpreter(model)
        )


class GUIController:
    def __init__(self, view):
        Facade().mode = "gui"
        self.data = Facade().data = {"main": []}
        self._model = GUIModel(self.data["main"])

        self._view = view
        self._interp = WfuzzInterpreter(self._model)

        # init gui
        self.start_gui()

        pub.subscribe(self.on_exit, "exit")

    def start_gui(self):
        self._view.start_gui(self)

    def on_exit(self, msg):
        print("oooo")



# Covered by GPL V2.0
# Coded by Carlos del Ojo Elias (deepbit@gmail.com)

import sys
import re

# python 2 and 3: iterator
from builtins import object


class TextParser(object):
    def __init__(self):
        self.string = ""
        self.oldindex = 0
        self.newindex = 0
        self.type = ""
        self.lastFull_line = None
        self.lastline = None

        self.actualIndex = 0

    def __del__(self):
        if self.type == "file":
            self.fd.close()

    def __str__(self):
        return str(self.matches)

    def __iter__(self):
        self.actualIndex = 0
        return self

    def __next__(self):
        try:
            value = self.matches[self.actualIndex]
            self.actualIndex += 1
            return value
        except Exception:
            raise StopIteration

    def setSource(self, t, *args):
        """Se especifica el tipo de entrada. Puede ser fichero o entrada estandard

        Ejemplos: setSource("file","/tmp/file")
                    setSource("stdin")\n"""

        if t == "file":
            self.type = t
            self.fd = open(args[0], "r")
        elif t == "stdin":
            if self.type == "file":
                self.fd.close()
            self.type = t
        elif t == "string":
            if self.type == "file":
                self.fd.close()
            self.type = t
            self.string = args[0]
            self.oldindex = 0
            self.newindex = 0
        else:
            print("Bad argument -- TextParser.setSource()\n")
            sys.exit(-1)

    def seekinit(self):
        self.oldindex = 0
        self.newindex = 0

    def readUntil(self, pattern, caseSens=True):
        "Lee lineas hasta que el patron (pattern) conincide en alguna linea"

        while True:
            if self.readLine() == 0:
                return False
            if self.search(pattern, caseSens) is True:
                break

        return True

    def search(self, pattern, caseSens=True, debug=0):
        "Intenta hacer Matching entre el pattern pasado por parametro y la ultima linea leida"

        if not caseSens:
            self.regexp = re.compile(pattern, re.IGNORECASE)
        else:
            self.regexp = re.compile(pattern)
        self.matches = self.regexp.findall(self.lastline)
        j = 0
        for i in self.matches:
            if not isinstance(i, tuple):
                self.matches[j] = tuple([self.matches[j]])
            j += 1

        #               DEBUG PARA MATCHING
        if debug == 1:
            print(("[", self.lastline, "-", pattern, "]"))
            print((len(self.matches)))
            print((self.matches))

        if len(self.matches) == 0:
            return False
        else:
            return True

    def __getitem__(self, key):
        "Para acceder a cada uno de los patrones que coinciden, esta preparado paragrupos de patrones, no para solo un patron"

        return self.matches[key]

    def skip(self, lines):
        "Salta las lines que se indiquen en el parametro"

        for i in range(lines):
            if self.readLine() == 0:
                return False

        return True

    def readLine(self):
        "Lee la siguiente linea eliminando retornos de carro"

        if self.type == "file":
            self.lastFull_line = self.fd.readline()
        elif self.type == "stdin":
            self.lastFull_line = input()
        elif self.type == "string":
            if self.newindex == -1:
                return 0

            if self.oldindex >= 0:
                self.newindex = self.string.find("\n", self.oldindex, len(self.string))
                if self.newindex == -1:
                    self.newindex = len(self.string) - 1
                self.lastFull_line = self.string[self.oldindex : self.newindex + 1]

                self.oldindex = self.newindex + 1
            else:
                self.lastFull_line = ""

        bytes_read = len(self.lastFull_line)

        s = self.lastFull_line
        self.lastline = s

        if s[-2:] == "\r\n":
            self.lastline = s[:-2]
        elif s[-1:] == "\r" or s[-1:] == "\n":
            self.lastline = s[:-1]

        return bytes_read



# Python 2 and 3: zip_longest
try:
    from itertools import zip_longest
except ImportError:
    from itertools import izip_longest as zip_longest

from ..helpers.obj_factory import ObjectFactory
from ..exception import FuzzExceptBadOptions
from ..facade import Facade
from ..dictionaries import (
    TupleIt,
    WrapperIt,
    SliceIt,
    EncodeIt,
    AllVarDictio,
)


class DictionaryFactory(ObjectFactory):
    def __init__(self):
        ObjectFactory.__init__(
            self,
            {
                "dictio_from_iterable": DictioFromIterableBuilder(),
                "dictio_from_payload": DictioFromPayloadBuilder(),
                "dictio_from_allvar": DictioFromAllVarBuilder(),
                "dictio_from_options": DictioFromOptions(),
            },
        )


class BaseDictioBuilder:
    @staticmethod
    def validate(options, selected_dic):
        if not selected_dic:
            raise FuzzExceptBadOptions("Empty dictionary! Check payload and filter")

        if len(selected_dic) == 1 and options["iterator"]:
            raise FuzzExceptBadOptions(
                "Several dictionaries must be used when specifying an iterator"
            )

    @staticmethod
    def get_dictio(options, selected_dic):
        if len(selected_dic) == 1:
            return TupleIt(selected_dic[0])
        elif options["iterator"]:
            return Facade().iterators.get_plugin(options["iterator"])(*selected_dic)
        else:
            return Facade().iterators.get_plugin("product")(*selected_dic)


class DictioFromIterableBuilder(BaseDictioBuilder):
    def __call__(self, options):
        selected_dic = []
        self._payload_list = []

        for d in [WrapperIt(x) for x in options["dictio"]]:
            selected_dic.append(d)

        self.validate(options, selected_dic)

        return self.get_dictio(options, selected_dic)


class DictioFromPayloadBuilder(BaseDictioBuilder):
    def __call__(self, options):
        selected_dic = []

        for payload in options["payloads"]:
            try:
                name, params, slicestr = [
                    x[0] for x in zip_longest(payload, (None, None, None))
                ]
            except ValueError:
                raise FuzzExceptBadOptions(
                    "You must supply a list of payloads in the form of [(name, {params}), ... ]"
                )

            if not params:
                raise FuzzExceptBadOptions(
                    "You must supply a list of payloads in the form of [(name, {params}), ... ]"
                )

            dictionary = Facade().payloads.get_plugin(name)(params)
            if "encoder" in params and params["encoder"] is not None:
                dictionary = EncodeIt(dictionary, params["encoder"])

            selected_dic.append(
                SliceIt(dictionary, slicestr) if slicestr else dictionary
            )

        self.validate(options, selected_dic)

        return self.get_dictio(options, selected_dic)


class DictioFromAllVarBuilder(BaseDictioBuilder):
    @staticmethod
    def from_all_fuzz_request_gen(options, dictio_list):
        for payload in dictio_list:
            if len(payload) > 1:
                raise FuzzExceptBadOptions(
                    "Only one payload is allowed when fuzzing all parameters!"
                )

            for var_name in options["compiled_seed"].history.wf_allvars_set.keys():
                yield (var_name, payload[0])

    def __call__(self, options):
        dictio_list = DictioFromOptions()(options)

        return AllVarDictio(
            self.from_all_fuzz_request_gen(options, dictio_list),
            dictio_list.count() * len(options["compiled_seed"].history.wf_allvars_set),
        )


class DictioFromOptions(BaseDictioBuilder):
    def __call__(self, options):
        if options["dictio"]:
            return DictioFromIterableBuilder()(options)
        else:
            return DictioFromPayloadBuilder()(options)


dictionary_factory = DictionaryFactory()



import re

# Python 2 and 3
try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

from wfuzz.plugin_api.mixins import DiscoveryPluginMixin
from wfuzz.plugin_api.base import BasePlugin
from wfuzz.plugin_api.urlutils import parse_url
from wfuzz.externals.moduleman.plugin import moduleman_plugin


KBASE_PARAM_PATH = "links.add_path"
KBASE_PARAM_ENQUEUE = "links.enqueue"
KBASE_PARAM_DOMAIN_REGEX = "links.domain"
KBASE_PARAM_REGEX = "links.regex"
KBASE_NEW_DOMAIN = "links.new_domains"


@moduleman_plugin
class links(BasePlugin, DiscoveryPluginMixin):
    name = "links"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    summary = "Parses HTML looking for new content."
    description = ("Parses HTML looking for new content",)
    category = ["active", "discovery"]
    priority = 99

    parameters = (
        ("enqueue", "True", False, "If True, enqueue found links.",),
        (
            "add_path",
            "False",
            False,
            "if True, re-enqueue found paths. ie. /path/link.html link enqueues also /path/",
        ),
        (
            "domain",
            None,
            False,
            "Regex of accepted domains tested against url.netloc. This is useful for restricting crawling certain domains.",
        ),
        (
            "regex",
            None,
            False,
            "Regex of accepted links tested against the full url. If domain is not set and regex is, domain defaults to .*. This is useful for restricting crawling certain file types.",
        ),
    )

    def __init__(self):
        BasePlugin.__init__(self)

        regex = [
            r'\b(?:(?<!data-)href)="((?!mailto:|tel:|#|javascript:).*?)"',
            r'\bsrc="((?!javascript:).*?)"',
            r'\baction="((?!javascript:).*?)"',
            r'<meta.*content="\d+;url=(.*?)">',  # http://en.wikipedia.org/wiki/Meta_refresh
            r'getJSON\("(.*?)"',
            r"[^/][`'\"]([\/][a-zA-Z0-9_.-]+)+(?!(?:[,;\s]))",  # based on https://github.com/nahamsec/JSParser/blob/master/handler.py#L93
        ]

        self.regex = []
        for regex_str in regex:
            self.regex.append(re.compile(regex_str, re.MULTILINE | re.DOTALL))

        self.regex_header = [
            ("Link", re.compile(r"<(.*)>;")),
            ("Location", re.compile(r"(.*)")),
        ]

        self.add_path = self._bool(self.kbase[KBASE_PARAM_PATH][0])
        self.enqueue_links = self._bool(self.kbase[KBASE_PARAM_ENQUEUE][0])

        self.domain_regex = None
        if self.kbase[KBASE_PARAM_DOMAIN_REGEX][0]:
            self.domain_regex = re.compile(
                self.kbase[KBASE_PARAM_DOMAIN_REGEX][0], re.IGNORECASE
            )

        self.regex_param = None
        if self.kbase[KBASE_PARAM_REGEX][0]:
            self.regex_param = re.compile(
                self.kbase[KBASE_PARAM_REGEX][0], re.IGNORECASE
            )

        if self.regex_param and self.domain_regex is None:
            self.domain_regex = re.compile(".*", re.IGNORECASE)

        self.list_links = set()

    def validate(self, fuzzresult):
        self.list_links = set()
        return fuzzresult.code in [200, 301, 302, 303, 307, 308]

    def process(self, fuzzresult):
        # <a href="www.owasp.org/index.php/OWASP_EU_Summit_2008">O
        # ParseResult(scheme='', netloc='', path='www.owasp.org/index.php/OWASP_EU_Summit_2008', params='', query='', fragment='')

        for header, regex in self.regex_header:
            if header in fuzzresult.history.headers.response:
                for link_url in regex.findall(
                    fuzzresult.history.headers.response[header]
                ):
                    if link_url:
                        self.process_link(fuzzresult, link_url)

        for regex in self.regex:
            for link_url in regex.findall(fuzzresult.history.content):
                if link_url:
                    self.process_link(fuzzresult, link_url)

    def process_link(self, fuzzresult, link_url):
        parsed_link = parse_url(link_url)

        if (
            not parsed_link.scheme
            or parsed_link.scheme == "http"
            or parsed_link.scheme == "https"
        ) and self.from_domain(fuzzresult, parsed_link):
            cache_key = parsed_link.cache_key(self.base_fuzz_res.history.urlp)
            if cache_key not in self.list_links:
                self.list_links.add(cache_key)
                self.enqueue_link(fuzzresult, link_url, parsed_link)

    def enqueue_link(self, fuzzresult, link_url, parsed_link):
        # dir path
        if self.add_path:
            split_path = parsed_link.path.split("/")
            newpath = "/".join(split_path[:-1]) + "/"
            self.queue_url(urljoin(fuzzresult.url, newpath))

        # file path
        new_link = urljoin(fuzzresult.url, link_url)

        if not self.regex_param or (
            self.regex_param and self.regex_param.search(new_link) is not None
        ):
            if self.enqueue_links:
                self.queue_url(new_link)
            self.add_result("link", "New link found", new_link)

    def from_domain(self, fuzzresult, parsed_link):
        # relative path
        if not parsed_link.netloc and parsed_link.path:
            return True

        # regex domain
        if (
            self.domain_regex
            and self.domain_regex.search(parsed_link.netloc) is not None
        ):
            return True

        # same domain
        if parsed_link.netloc == self.base_fuzz_res.history.urlp.netloc:
            return True

        if (
            parsed_link.netloc
            and parsed_link.netloc not in self.kbase[KBASE_NEW_DOMAIN]
        ):
            self.kbase[KBASE_NEW_DOMAIN].append(parsed_link.netloc)
            self.add_result(
                "domain", "New domain found (link not enqueued)", parsed_link.netloc
            )



from collections import defaultdict


class HttpCache:
    def __init__(self):
        # cache control
        self.__cache_map = defaultdict(list)

    def update_cache(self, req, category="default"):
        key = req.to_cache_key()

        # first hit
        if key not in self.__cache_map:
            self.__cache_map[key].append(category)
            return True
        elif key in self.__cache_map and category not in self.__cache_map[key]:
            self.__cache_map[key].append(category)
            return True

        return False

    def msg_in_cache(self, req, category="default"):
        key = req.to_cache_key()

        return key in self.__cache_map and category in self.__cache_map[key]



import time
import hashlib
import re
import itertools
from enum import Enum

from threading import Lock
from collections import defaultdict, namedtuple

from .filters.ppfilter import FuzzResFilter
from .facade import ERROR_CODE

from .helpers.str_func import python2_3_convert_to_unicode
from .helpers.obj_dyn import rgetattr
from .helpers.utils import MyCounter
from .helpers.obj_dic import DotDict


FuzzWord = namedtuple("FuzzWord", ["content", "type"])


class FuzzWordType(Enum):
    WORD, FUZZRES = range(2)


class FuzzType(Enum):
    (SEED, BACKFEED, RESULT, ERROR, STARTSEED, ENDSEED, CANCEL, PLUGIN,) = range(8)


class FuzzItem(object):
    newid = itertools.count(0)

    def __init__(self, item_type):
        self.item_id = next(FuzzItem.newid)
        self.item_type = item_type
        self.rlevel = 1
        self.discarded = False

    def __str__(self):
        return "FuzzItem, type: {}".format(self.item_type.name)

    def __lt__(self, other):
        return self.item_id < other.item_id

    def __le__(self, other):
        return self.item_id <= other.item_id

    def __gt__(self, other):
        return self.item_id > other.item_id

    def __ge__(self, other):
        return self.item_id >= other.item_id

    def __eq__(self, other):
        return self.item_id == other.item_id

    def __ne__(self, other):
        return self.item_id != other.item_id


class FuzzStats:
    def __init__(self):
        self.mutex = Lock()

        self.url = ""
        self.seed = None

        self.total_req = 0
        self.pending_fuzz = MyCounter()
        self.pending_seeds = MyCounter()
        self.processed = MyCounter()
        self.backfeed = MyCounter()
        self.filtered = MyCounter()

        self.totaltime = 0
        self.__starttime = 0

        self._cancelled = False

    @staticmethod
    def from_options(options):
        tmp_stats = FuzzStats()

        tmp_stats.url = options["compiled_seed"].history.redirect_url
        tmp_stats.total_req = options["compiled_dictio"].count()
        tmp_stats.seed = options["compiled_seed"]

        return tmp_stats

    def get_stats(self):
        return {
            "url": self.url,
            "total": self.total_req,
            "backfed": self.backfeed(),
            "processed": self.processed(),
            "pending": self.pending_fuzz(),
            "filtered": self.filtered(),
            "pending_seeds": self.pending_seeds(),
            "totaltime": time.time() - self.__starttime,
        }

    def mark_start(self):
        with self.mutex:
            self.__starttime = time.time()

    def mark_end(self):
        with self.mutex:
            self.totaltime = time.time() - self.__starttime

    @property
    def cancelled(self):
        with self.mutex:
            return self._cancelled

    @cancelled.setter
    def cancelled(self, v):
        with self.mutex:
            self._cancelled = v

    def __str__(self):
        string = ""

        string += "Total time: %s\n" % str(self.totaltime)[:8]

        if self.backfeed() > 0:
            string += "Processed Requests: %s (%d + %d)\n" % (
                str(self.processed())[:8],
                (self.processed() - self.backfeed()),
                self.backfeed(),
            )
        else:
            string += "Processed Requests: %s\n" % (str(self.processed())[:8])
        string += "Filtered Requests: %s\n" % (str(self.filtered())[:8])
        string += (
            "Requests/sec.: %s\n"
            % str(self.processed() / self.totaltime if self.totaltime > 0 else 0)[:8]
        )

        return string

    def update(self, fuzzstats2):
        self.url = fuzzstats2.url
        self.total_req += fuzzstats2.total_req
        self.totaltime += fuzzstats2.totaltime

        self.backfeed._operation(fuzzstats2.backfeed())
        self.processed._operation(fuzzstats2.processed())
        self.pending_fuzz._operation(fuzzstats2.pending_fuzz())
        self.filtered._operation(fuzzstats2.filtered())
        self.pending_seeds._operation(fuzzstats2.pending_seeds())


class FuzzPayload:
    def __init__(self):
        self.marker = None
        self.word = None
        self.index = None
        self.field = None
        self.content = None
        self.is_baseline = False
        self.type = None

    @property
    def value(self):
        if self.content is None:
            return None
        return (
            self.content
            if self.field is None
            else str(rgetattr(self.content, self.field))
        )

    def description(self):
        if self.is_baseline:
            return self.content

        if self.marker is None:
            return ""

        # return default value
        if self.field is None and isinstance(self.content, FuzzResult):
            return self.content.url
        elif self.field is not None and isinstance(self.content, FuzzResult):
            return str(rgetattr(self.content, self.field))

        return self.value

    def __str__(self):
        return "type: {} index: {} marker: {} content: {} field: {} value: {}".format(
            self.type,
            self.index,
            self.marker,
            self.content.__class__,
            self.field,
            self.value,
        )


class FPayloadManager:
    def __init__(self):
        self.payloads = defaultdict(list)

    def add(self, payload_dict, fuzzword=None, is_baseline=False):
        fp = FuzzPayload()
        fp.marker = payload_dict["full_marker"]
        fp.word = payload_dict["word"]
        fp.index = (
            int(payload_dict["index"]) if payload_dict["index"] is not None else 1
        )
        fp.field = payload_dict["field"]
        fp.content = fuzzword.content if fuzzword else None
        fp.type = fuzzword.type if fuzzword else None
        fp.is_baseline = is_baseline

        self.payloads[fp.index].append(fp)

    def update_from_dictio(self, dictio_item):
        for index, dictio_payload in enumerate(dictio_item, 1):
            fuzz_payload = None
            for fuzz_payload in self.payloads[index]:
                fuzz_payload.content = dictio_payload.content
                fuzz_payload.type = dictio_payload.type

            # payload generated not used in seed but in filters
            if fuzz_payload is None:
                self.add(
                    {"full_marker": None, "word": None, "index": index, "field": None},
                    dictio_item[index - 1],
                )

    def get_fuzz_words(self):
        return [payload.word for payload in self.get_payloads()]

    def get_payload(self, index):
        return self.payloads[index]

    def get_payload_type(self, index):
        return self.get_payload(index)[0].type

    def get_payload_content(self, index):
        return self.get_payload(index)[0].content

    def get_payloads(self):
        for index, elem_list in sorted(self.payloads.items()):
            for elem in elem_list:
                yield elem

    def description(self):
        payl_descriptions = [payload.description() for payload in self.get_payloads()]
        ret_str = " - ".join([p_des for p_des in payl_descriptions if p_des])

        return ret_str

    def __str__(self):
        return "\n".join([str(payload) for payload in self.get_payloads()])


class FuzzError(FuzzItem):
    def __init__(self, exception):
        FuzzItem.__init__(self, FuzzType.ERROR)
        self.exception = exception


class FuzzResult(FuzzItem):
    newid = itertools.count(0)
    FUZZRESULT_SHARED_FILTER = FuzzResFilter()

    def __init__(self, history=None, exception=None, track_id=True):
        FuzzItem.__init__(self, FuzzType.RESULT)
        self.history = history

        self.exception = exception
        self.is_baseline = False
        self.rlevel_desc = ""
        self.nres = next(FuzzResult.newid) if track_id else 0

        self.chars = 0
        self.lines = 0
        self.words = 0
        self.md5 = ""

        self.update()

        self.plugins_res = []

        self.payload_man = None

        self._fields = None
        self._show_field = False

    @property
    def plugins(self):
        dic = defaultdict(lambda: defaultdict(list))

        for pl in self.plugins_res:
            if pl.source == FuzzPlugin.OUTPUT_SOURCE:
                continue
            dic[pl.source][pl.itype].append(pl.data)

        ret = DotDict()
        for key, first in dic.items():
            ret[key] = DotDict()
            for seckey, second in first.items():
                ret[key][seckey] = second

        return ret

    def update(self, exception=None):
        self.item_type = FuzzType.RESULT
        if exception:
            self.exception = exception

        if self.history and self.history.content:
            m = hashlib.md5()
            m.update(python2_3_convert_to_unicode(self.history.content))
            self.md5 = m.hexdigest()

            self.chars = len(self.history.content)
            self.lines = self.history.content.count("\n")
            self.words = len(re.findall(r"\S+", self.history.content))

        return self

    def __str__(self):
        res = '%05d:  C=%03d   %4d L\t   %5d W\t  %5d Ch\t  "%s"' % (
            self.nres,
            self.code,
            self.lines,
            self.words,
            self.chars,
            self.description,
        )
        for plugin in self.plugins_res:
            if plugin.itype == FuzzPlugin.SUMMARY_ITYPE:
                res += "\n  |_ %s" % plugin.issue

        return res

    @property
    def description(self):
        res_description = self.payload_man.description() if self.payload_man else None

        if not res_description:
            res_description = self.url

        ret_str = None

        if self._show_field is True:
            ret_str = self._field()
        elif self._show_field is False and self._fields is not None:
            ret_str = "{} | {}".format(res_description, self._field())
        else:
            ret_str = res_description

        if self.exception:
            return ret_str + "! " + str(self.exception)

        if self.rlevel > 1:
            return self.rlevel_desc + " - " + ret_str

        return ret_str

    def eval(self, expr):
        return self.FUZZRESULT_SHARED_FILTER.is_visible(self, expr)

    def _field(self, separator=", "):
        list_eval = [self.eval(field) for field in self._fields]
        return " | ".join(
            [
                separator.join(el) if isinstance(el, list) else str(el)
                for el in list_eval
            ]
        )

    # parameters in common with fuzzrequest
    @property
    def content(self):
        return self.history.content if self.history else ""

    @property
    def url(self):
        return self.history.url if self.history else ""

    @property
    def code(self):
        if self.history and self.history.code >= 0 and not self.exception:
            return int(self.history.code)
        # elif not self.history.code:
        # return 0
        else:
            return ERROR_CODE

    @property
    def timer(self):
        return self.history.reqtime if self.history and self.history.reqtime else 0

    # factory methods

    def update_from_options(self, options):
        self._fields = options["fields"]
        self._show_field = options["show_field"]


class FuzzPlugin(FuzzItem):
    OUTPUT_SOURCE = "output"
    SUMMARY_ITYPE = "summary"
    NONE, INFO, LOW, MEDIUM, HIGH, CRITICAL = range(6)
    MIN_VERBOSE = INFO

    def __init__(self):
        FuzzItem.__init__(self, FuzzType.PLUGIN)
        self.source = ""
        self.issue = ""
        self.itype = ""
        self.data = ""
        self._exception = None
        self._seed = None
        self.severity = self.INFO

    def is_visible(self, verbose):
        if verbose and self.itype == self.SUMMARY_ITYPE:
            return False

        if not verbose and self.severity >= self.MIN_VERBOSE:
            return False

        return True



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.exception import FuzzExceptBadFile, FuzzExceptBadOptions
from wfuzz.fuzzobjects import FuzzResult
from wfuzz.fuzzrequest import FuzzRequest
from wfuzz.plugin_api.base import BasePayload
from wfuzz.helpers.obj_dyn import rgetattr
from wfuzz.fuzzobjects import FuzzWordType


import datetime
import string
import re
import struct
import zipfile

TAG = re.compile(r"</?(\w*)>", re.M)  # Match a XML tag
nvprint = string.printable.replace("\x0b", "").replace("\x0c", "")  # Printables


@moduleman_plugin
class burpstate(BasePayload):
    name = "burpstate"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.1"
    description = (
        "*ALERT*: https://portswigger.net/blog/goodbye-state-files-we-wont-miss-you",
        "",
        "Returns fuzz results' from a Burp saved state file. This payload's code is based on burp2xml.py:",
        "Developed by Paul Haas, <phaas AT redspin DOT com> under Redspin. Inc.",
        "Licensed under the GNU Public License version 3.0 (2008-2009)",
        "Process Burp Suite Professional's output into a well-formed XML document.",
        "",
        "Currently, the whole burp state file is read, in the future this needs to be changed to be more memory efficient.",
    )
    summary = "Returns fuzz results from a Burp state."
    category = ["default"]
    priority = 99

    parameters = (
        ("fn", "", True, "Filename of a valid Burp state file."),
        (
            "attr",
            None,
            False,
            "Fuzzresult attribute to return. If not specified the whole object is returned.",
        ),
        (
            "source",
            "proxy, target",
            False,
            "A list of separated Burp sources to get the HTTP requests and responses from. It could be proxy or target tool.",
        ),
        (
            "checkversion",
            False,
            False,
            "If the Burp log file version is unknown an exception will be raised and execution will fail. Checked with burp state file version 65, 67.",
        ),
    )

    default_parameter = "fn"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        self.__max = -1
        self.attr = self.params["attr"]
        self._it = self.burp_to_xml(self.params["fn"])

        if any(i not in ["proxy", "target"] for i in self.params["source"].split(",")):
            raise FuzzExceptBadOptions("Unknown burp source parameter")

        self.request_tags = []
        self.response_tags = []

        if "proxy" in self.params["source"]:
            self.request_tags.append("</originalRequest>")
            self.response_tags.append("</originalResponse>")

        if "target" in self.params["source"]:
            self.request_tags.append("</request>")
            self.response_tags.append("</response>")

    def __iter__(self):
        return self

    def count(self):
        return self.__max

    def get_type(self):
        return FuzzWordType.FUZZRES if not self.attr else FuzzWordType.WORD

    def get_next(self):
        next_item = next(self._it)

        return next_item if not self.attr else rgetattr(next_item, self.attr)

    def milliseconds_to_date(self, milliseconds):
        """Convert milliseconds since Epoch (from Java) to Python date structure:
        See: http://java.sun.com/j2se/1.4.2/docs/api/java/util/Date.html

        There is no direct way to convert milliseconds since Epoch to Python object
        So we convert the milliseconds to seconds first as a POSIX timestamp which
        can be used to get a valid date, and then use the parsed values from that
        object along with converting mili -> micro seconds in a new date object."""
        try:
            d = datetime.datetime.fromtimestamp(milliseconds / 1000)
            date = datetime.datetime(
                d.year,
                d.month,
                d.day,
                d.hour,
                d.minute,
                d.second,
                (milliseconds % 1000) * 1000,
            )
        except ValueError:  # Bad date, just return the milliseconds
            date = str(milliseconds)
            return None
        return date

    def burp_binary_field(self, field, i):
        """Strip Burp Suite's binary format characters types from our data.
        The first character after the leading tag describes the type of the data."""
        if len(field) <= i:
            return None, -1
        elif field[i] == "\x00":  # 4 byte integer value
            return str(struct.unpack(">I", field[i + 1 : i + 5])[0]), 5
        elif field[i] == "\x01":  # Two possible unsigned long long types
            if field[i + 1] == "\x00":  # (64bit) 8 Byte Java Date
                ms = struct.unpack(">Q", field[i + 1 : i + 9])[0]
                date = self.milliseconds_to_date(ms)
                value = (
                    date.ctime() if date else 0
                )  # Use the ctime string format for date
            else:  # Serial Number only used ocasionally in Burp
                value = str(struct.unpack(">Q", field[i + 1 : i + 9])[0])
            return value, 9
        elif field[i] == "\x02":  # Boolean Object True/False
            return str(struct.unpack("?", field[i + 1 : i + 2])[0]), 2
        elif field[i] == "\x03" or field[i] == "\x04":  # 4 byte length + string
            length = struct.unpack(">I", field[i + 1 : i + 5])[0]
            # print "Saw string of length", length, "at", i + 5, i + 5+length
            value = field[i + 5 : i + 5 + length]
            if "<" in value or ">" in value or "&" in value:  # Sanatize HTML w/CDATA
                value = "<![CDATA[" + value.replace("]]>", "]]><![CDATA[") + "]]>"
            value = "".join(c for c in value if c in nvprint)  # Remove nonprintables
            return value, 5 + length  # ** TODO: Verify length by matching end tag **
        print("Unknown binary format", repr(field[i]))
        return None, -1

    def strip_cdata(self, data):
        if data.startswith("<![CDATA["):
            data = data[9:]

        if data.endswith("]]>"):
            data = data[:-3]

        return data

    def burp_to_xml(self, filename):
        """Unzip Burp's file, remove non-printable characters, CDATA any HTML,
        include a valid XML header and trailer, and return a valid XML string."""

        z = zipfile.ZipFile(self.find_file(filename))  # Open Burp's zip file
        burp = z.read("burp", "rb")  # Read-in the main burp file
        m = TAG.match(burp, 0)  # Match a tag at the start of the string
        while m:
            index = m.end()
            etag = m.group().replace("<", "</")  # Matching tag

            m = TAG.match(burp, index)  # Attempt to get the next tag
            if not m:  # Data folows
                # Read the type of data using Burp's binary data headers
                value, length = self.burp_binary_field(burp, index)
                if value is None:
                    break

                index += length + len(etag)  # Point our index to the next tag
                m = TAG.match(burp, index)  # And retrieve it

                if (
                    self.params["checkversion"]
                    and etag == "</version>"
                    and value not in ["65", "67"]
                ):
                    raise FuzzExceptBadFile("Unknown burp log version %s" % value)

                if etag == "</https>":
                    https_tag = value == "True"

                if etag in self.request_tags:
                    raw_request = self.strip_cdata(value)

                if etag in self.response_tags:
                    fr = FuzzRequest()
                    fr.update_from_raw_http(
                        raw_request,
                        "http" if not https_tag else "https",
                        self.strip_cdata(value),
                    )
                    frr = FuzzResult(history=fr)

                    raw_request = ""
                    https_tag = ""

                    yield frr.update()



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.plugin_api.base import BasePayload
from wfuzz.fuzzobjects import FuzzWordType


@moduleman_plugin
class buffer_overflow(BasePayload):
    name = "buffer_overflow"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.2"
    description = ()
    summary = "Returns a string using the following pattern A * given number."
    category = ["default"]
    priority = 99

    parameters = (("size", "", True, "Size of the overflow string."),)

    default_parameter = "size"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        self.bov_list = ["A" * int(self.params["size"])]
        self.current = 0

    def count(self):
        return 1

    def get_next(self):
        if self.current == 0:
            elem = self.bov_list[self.current]
            self.current += 1
            return elem
        else:
            raise StopIteration

    def get_type(self):
        return FuzzWordType.WORD



import os


# Python 2 and 3
import sys

if sys.version_info >= (3, 0):
    from urllib.parse import ParseResult
    from urllib.parse import urlparse
    from urllib.parse import parse_qs
else:
    from urlparse import ParseResult
    from urlparse import urlparse
    from urlparse import parse_qs

from wfuzz.facade import Facade
from wfuzz.exception import FuzzExceptBadAPI


class FuzzRequestParse(ParseResult):
    @property
    def ffname(self):
        """
        Returns script plus extension from an URL. ie. http://www.localhost.com/kk/index.html?id=3
        will return index.html
        """
        u = self.path.split("/")[-1:][0]

        return u

    @property
    def fext(self):
        """
        Returns script extension from an URL. ie. http://www.localhost.com/kk/index.html?id=3
        will return .html
        """
        return os.path.splitext(self.ffname)[1]

    @property
    def fname(self):
        """
        Returns script name from an URL. ie. http://www.localhost.com/kk/index.html?id=3
        will return index
        """
        return os.path.splitext(self.ffname)[0]

    @property
    def isbllist(self):
        fext = self.fext
        return fext != "." and fext in Facade().sett.get(
            "kbase", "discovery.blacklist"
        ).split("-")

    @property
    def hasquery(self):
        return self.query != ""

    def cache_key(self, base_urlp=None):
        scheme = self.scheme
        netloc = self.netloc

        if base_urlp:
            scheme = self.scheme if self.scheme else base_urlp.scheme
            netloc = self.netloc if self.netloc else base_urlp.netloc

        key = "{}-{}-{}-{}".format(scheme, netloc, self.path, self.params)
        dicc = {"g{}".format(key): True for key in parse_qs(self.query).keys()}

        # take URL parameters into consideration
        url_params = list(dicc.keys())
        url_params.sort()
        key += "-" + "-".join(url_params)

        return key


def parse_url(url):
    # >>> urlparse.urlparse("http://some.page.pl/nothing.py;someparam=some;otherparam=other?query1=val1&query2=val2#frag")
    # ParseResult(scheme='http', netloc='some.page.pl', path='/nothing.py', params='someparam=some;otherparam=other', query='query1=val1&query2=val2', fragment='frag')

    scheme, netloc, path, params, query, fragment = urlparse(url)
    return FuzzRequestParse(scheme, netloc, path, params, query, fragment)


def check_content_type(fuzzresult, which):
    ctype = None
    if "Content-Type" in fuzzresult.history.headers.response:
        ctype = fuzzresult.history.headers.response["Content-Type"]

    if which == "text":
        return not ctype or (
            ctype and any([ctype.find(x) >= 0 for x in ["text/plain"]])
        )
    else:
        raise FuzzExceptBadAPI("Unknown content type")



try:
    from collections.abc import Callable
except ImportError:
    from collections import Callable


def moduleman_plugin(*args):
    method_args = []

    def inner_decorator(cls):
        for method in method_args:
            if not (method in dir(cls)):
                raise Exception("Required method %s not implemented" % method)
        cls.__PLUGIN_MODULEMAN_MARK = "Plugin mark"

        return cls

    if not isinstance(args[0], Callable):
        method_args += args
        return inner_decorator

    return inner_decorator(args[0])



from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.plugin_api.payloadtools import BingIter
from wfuzz.plugin_api.base import BasePayload
from wfuzz.fuzzobjects import FuzzWordType


@moduleman_plugin
class bing(BasePayload):
    name = "bing"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.2"
    description = (
        'intitle:"JBoss JMX Management Console"',
        "Some examples of bing hacking:",
        "http://www.elladodelmal.com/2010/02/un-poco-de-bing-hacking-i-de-iii.html",
    )

    summary = "Returns URL results of a given bing API search (needs api key)."
    category = ["default"]
    priority = 99

    parameters = (
        ("dork", "", True, "Google dork search string."),
        ("offset", "0", False, "Offset index, starting at zero."),
        ("limit", "0", False, "Number of results. Zero for all."),
    )

    default_parameter = "dork"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        offset = int(params["offset"])
        limit = int(params["limit"])

        self._it = BingIter(params["dork"], offset, limit)

    def count(self):
        return self._it.max_count

    def get_next(self):
        return next(self._it)

    def get_type(self):
        return FuzzWordType.WORD



import re
import base64

from wfuzz.exception import FuzzExceptBadFile
from wfuzz.fuzzobjects import FuzzResult, FuzzWordType
from wfuzz.fuzzrequest import FuzzRequest
from wfuzz.plugin_api.base import BasePayload
from wfuzz.externals.moduleman.plugin import moduleman_plugin
from wfuzz.helpers.obj_dyn import rgetattr


@moduleman_plugin
class autorize(BasePayload):
    name = "autorize"
    author = ("Xavi Mendez (@xmendez)",)
    version = "0.2"
    description = ("Reads burp extension autorize states",)
    summary = "Returns fuzz results' from autorize."
    category = ["default"]
    priority = 99

    parameters = (
        ("fn", "", True, "Filename of a valid autorize state file."),
        (
            "attr",
            None,
            False,
            "Attribute of fuzzresult to return. If not specified the whole object is returned.",
        ),
    )

    default_parameter = "fn"

    def __init__(self, params):
        BasePayload.__init__(self, params)

        self.__max = -1
        self.attr = self.params["attr"]
        self._it = self._gen_wfuzz(self.params["fn"])

    def count(self):
        return self.__max

    def get_next(self):
        next_item = next(self._it)

        return next_item if not self.attr else rgetattr(next_item, self.attr)

    def get_type(self):
        return FuzzWordType.WORD

    def _gen_wfuzz(self, output_fn):
        try:

            with open(self.find_file(output_fn), "r") as f:
                for (
                    url1,
                    port1,
                    schema1,
                    req1,
                    resp1,
                    url2,
                    port2,
                    schema2,
                    req2,
                    resp2,
                    url3,
                    port3,
                    schema3,
                    req3,
                    resp3,
                    res1,
                    res2,
                ) in [re.split(r"\t+", x) for x in f.readlines()]:
                    raw_req1 = base64.decodestring(req2)
                    # raw_res1 = base64.decodestring(res2)

                    item = FuzzResult()
                    item.history = FuzzRequest()
                    item.history.update_from_raw_http(raw_req1, schema1)

                    yield item
        except IOError as e:
            raise FuzzExceptBadFile("Error opening wfuzz payload file. %s" % str(e))
        except EOFError:
            raise StopIteration
